/*
Copyright (c) 2011 Filippo Mariani

This software is provided 'as-is', without any express or implied warranty.
In no event will the authors be held liable for any damages arising from the use of this software.
Permission is granted to anyone to use this software for any purpose, 
including commercial applications, and to alter it and redistribute it freely, 
subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.
*/
#ifndef BT_ALLOW_ARRAY_COPY_OPERATOR
	#define BT_ALLOW_ARRAY_COPY_OPERATOR
#endif //BT_ALLOW_ARRAY_COPY_OPERATOR

#include <LinearMath/btAlignedObjectArray.h>
#include <BulletCollision/CollisionShapes/btTriangleIndexVertexArray.h>
#include "btTriangleMeshEx.h"

#include "stb_image.h"
#include <string>
#include <vector>
#include <map>

#define MAX_NUMBER_SUBMESHES 500



 //region GL_STUFF
#define GL_BGR                            0x80E0
#define GL_BGRA                           0x80E1
#define GL_CLAMP_TO_EDGE                  0x812F
#define GL_GENERATE_MIPMAP                0x8191

#ifndef GL_ARB_texture_compression
#define GL_COMPRESSED_ALPHA_ARB           0x84E9
#define GL_COMPRESSED_LUMINANCE_ARB       0x84EA
#define GL_COMPRESSED_LUMINANCE_ALPHA_ARB 0x84EB
#define GL_COMPRESSED_INTENSITY_ARB       0x84EC
#define GL_COMPRESSED_RGB_ARB             0x84ED
#define GL_COMPRESSED_RGBA_ARB            0x84EE
#define GL_TEXTURE_COMPRESSION_HINT_ARB   0x84EF
#define GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB 0x86A0
#define GL_TEXTURE_COMPRESSED_ARB         0x86A1
#define GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB 0x86A2
#define GL_COMPRESSED_TEXTURE_FORMATS_ARB 0x86A3
#endif
//endregion

#ifdef __GNUC__
#include <string.h>
#endif //__GNUC__



 //region DisplayListHelper
btAlignedObjectArray< GLuint > DisplayListHelper::g_displayListArray;
void DisplayListHelper::ClearArray()	{
	for (int i=0;i< g_displayListArray.size();i++)	{
		const GLuint dl = g_displayListArray[i];
		if (dl>0) glDeleteLists(dl,1);
	}
	g_displayListArray.resize(0);	
}
void DisplayListHelper::Add(GLuint dl)	{
	if (dl==0) return;
	for (int i=0,sz = g_displayListArray.size();i< sz;i++) {
		if (dl==g_displayListArray[i]) return;
	}
	g_displayListArray.push_back(dl);
}
bool DisplayListHelper::IsValid(GLuint dl)	{
	if (dl==0) return false;
	for (int i=0,sz = g_displayListArray.size();i< sz;i++) {
		if (dl==g_displayListArray[i]) return true;
	}
	return false;
}
//endregion
 //region btTriangleMeshExFootPrint::Helper
btAlignedObjectArray< const btTriangleMeshExFootPrint* > btTriangleMeshExFootPrint::Helper::g_triangleMeshExFootPrintArray;
void btTriangleMeshExFootPrint::Helper::ClearArray()	{
	for (int i=0;i< g_triangleMeshExFootPrintArray.size();i++)	{
		btTriangleMeshExFootPrint* dl = const_cast < btTriangleMeshExFootPrint* > (g_triangleMeshExFootPrintArray[i]);
		if (dl) {dl->resize(0);delete dl;dl=NULL;}
	}
	g_triangleMeshExFootPrintArray.resize(0);	
}
void btTriangleMeshExFootPrint::Helper::Add(const btTriangleMeshExFootPrint* dl)	{
	if (!dl) return;
	for (int i=0,sz = g_triangleMeshExFootPrintArray.size();i< sz;i++) {
		if (dl==g_triangleMeshExFootPrintArray[i]) return;
	}
	g_triangleMeshExFootPrintArray.push_back(dl);
}
bool btTriangleMeshExFootPrint::Helper::IsValid(const btTriangleMeshExFootPrint* dl)	{
	if (!dl) return false;
	for (int i=0,sz = g_triangleMeshExFootPrintArray.size();i< sz;i++) {
		if (dl==g_triangleMeshExFootPrintArray[i]) return true;
	}
	return false;
}
//endregion
 //region TextureHelper
std::map < std::string, GLuint > textureMap;
btAlignedObjectArray< GLuint > TextureHelper::g_textureArray;
void TextureHelper::ClearArray()	{
	for (int i=0;i< g_textureArray.size();i++)	{
		const GLuint dl = g_textureArray[i];
		if (dl>0) glDeleteTextures(1,&dl);
	}
	g_textureArray.resize(0);
	textureMap.clear();	
}
void TextureHelper::Add(GLuint dl)	{
	if (dl==0) return;
	for (int i=0,sz = g_textureArray.size();i< sz;i++) {
		if (dl==g_textureArray[i]) return;
	}
	g_textureArray.push_back(dl);
}
bool TextureHelper::IsValid(GLuint dl)	{
	if (dl==0) return false;
	for (int i=0,sz = g_textureArray.size();i< sz;i++) {
		if (dl==g_textureArray[i]) return false;
	}
	return true;
}
bool FileExists(const char* path);
GLuint TextureHelper::Get(const char* filename)	{
	if (!filename || strlen(filename)==0) return 0;
	std::map < std::string, GLuint >::const_iterator it = textureMap.find(filename);
	if (it!=textureMap.end()) return it->second;
	if (!FileExists(filename)) {
		printf("Error in TextureHelper::Get(...). File: \"%s\" does not exist.",filename);
		return 0;		
	}	
	const bool textureCompression = true;//false;	// TODO: add basic extension support to check for it
	
	int GL_KIND_MIN=-1;
	int GL_KIND_MAX=-1;
	int GL_KIND_WRAPS=-1;
	int GL_KIND_WRAPT=-1;
			
	int x,y,n;
	int wantedImageChannels = 0;
	unsigned char *data = stbi_load(filename, &x, &y, &n, wantedImageChannels);
	if (!data) {
		printf("Error. Can't load image file: %s \n",filename);
		stbi_image_free(data);	// Needed ?
		return 0;
	}
	if (wantedImageChannels>0) n = wantedImageChannels;
	const bool flipImageVertically = true;
	if (flipImageVertically)	{
		const int lineByteWidth = x*n;
		std::vector< unsigned char > tempLine(lineByteWidth);
		for (int i = 0; i < y/2; i++)	{
			memcpy(&tempLine[0],&data[i*lineByteWidth],lineByteWidth);
			memcpy(&data[i*lineByteWidth],&data[(y-i-1)*lineByteWidth],lineByteWidth);
			memcpy(&data[(y-i-1)*lineByteWidth],&tempLine[0],lineByteWidth);			
		}
	}	
	const bool invertRB = false;
	if (invertRB && n>=3) {
		unsigned char t;
		const int totalByteLength = x*y*n;
		for (int i = 0; i < totalByteLength; i=i+n)	{
			unsigned char& r = data[i];
			unsigned char& b = data[i+2];			
			t = r;	r = b;	b = t;
		}
	}
	
	GLuint texId = 0;
   	
	//Make a texture out of data
	const int numOfOutputComponents = n;
	{
	const int byteAlignement = 1;
 	glPixelStorei(GL_UNPACK_ALIGNMENT, byteAlignement);

   	glGenTextures(1,&texId);		

    glBindTexture(GL_TEXTURE_2D, texId);
 	
    if (textureCompression && numOfOutputComponents>2) {
		int FORMAT=GL_COMPRESSED_RGB_ARB;
		#ifdef _BIG_ENDIAN					// TO TEST OR REMOVE
		FORMAT =  GL_COMPRESSED_BGR_ARB;;
		#endif	
		if (numOfOutputComponents==4) 	{
			FORMAT=GL_COMPRESSED_RGBA_ARB;
			//FORMAT=GL_COMPRESSED_RGBA_S3TC_DXT5_EXT;
			#ifdef _BIG_ENDIAN					// TO TEST OR REMOVE
			FORMAT =  GL_COMPRESSED_BGRA_ARB;;
			#endif
		}
		if (numOfOutputComponents==3) {
			gluBuild2DMipmaps(GL_TEXTURE_2D, FORMAT, 
			         		  			x,y,GL_RGB, GL_UNSIGNED_BYTE, 
			         		  			&data[0]);
		}	         		  			
		else gluBuild2DMipmaps(GL_TEXTURE_2D, FORMAT, 
			         		  x,y, GL_RGBA, GL_UNSIGNED_BYTE, 
					          &data[0]);	
	//COMPONENTS: components: Specifies the number of color components in the texture. Must be 1, 2, 3, or 4. 
	//FORMAT: GL_COLOR_INDEX, GL_RED, GL_GREEN, GL_BLUE, GL_ALPHA, GL_RGB, GL_RGBA, GL_LUMINANCE, and GL_LUMINANCE_ALPHA.					          				          
	}
	else	//NO TEXTURE COMPRESSION
	{
		int FORMAT=GL_BGR;//GL_RGB;//_EXT;
		#ifdef _BIG_ENDIAN					// TO TEST OR REMOVE
		FORMAT =  GL_BGR_EXT;;
		#endif		
		if (numOfOutputComponents==4) {
			FORMAT=GL_BGRA;
			//FORMAT=GL_RGBA;//_EXT;
			#ifdef _BIG_ENDIAN				// TO TEST OR REMOVE
			FORMAT =  GL_BGRA_EXT;;
			#endif			
		}	
		else if (numOfOutputComponents==1) {
			FORMAT=GL_ALPHA;	//GL_ALPHA or GL_LUMINANCE ?
			//MessageBoxA(NULL,"OK","",MB_OK);
		}	

		gluBuild2DMipmaps(GL_TEXTURE_2D, numOfOutputComponents, x,y,
			         		  FORMAT, GL_UNSIGNED_BYTE,&data[0]);
		//COMPONENTS: components: Specifies the number of color components in the texture. Must be 1, 2, 3, or 4. 	
		//FORMAT: GL_COLOR_INDEX, GL_RED, GL_GREEN, GL_BLUE, GL_ALPHA, GL_RGB, GL_RGBA, GL_LUMINANCE, and GL_LUMINANCE_ALPHA.
		// I don't know which is correct!!! I'm porting my old code and I'm using GL_BGR_EXT and GL_BGRA_EXT that I cannot find anymore in
		//	openGL docs!!!					          
	}    

	if (GL_KIND_MIN<0) GL_KIND_MIN=GL_LINEAR_MIPMAP_LINEAR;
	if (GL_KIND_MAX<0) GL_KIND_MAX=GL_LINEAR;
	if (GL_KIND_WRAPS<0) GL_KIND_WRAPS=GL_REPEAT;
	if (GL_KIND_WRAPT<0) GL_KIND_WRAPT=GL_REPEAT;

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_KIND_MAX);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_KIND_MIN);

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_KIND_WRAPS);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_KIND_WRAPT);
	glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE); // automatic mipmap
  
   
   	textureMap[filename] = texId;
	Add(texId);
	
	}	
	stbi_image_free(data);
	printf("Loaded texture: \"%s\"\n",filename);
	//printf("Loaded texture: \"%s\" (%dx%dx%d)\n",filename,x,y,numOfOutputComponents);
	return texId;
}
//endregion
 //region Material
static void clampFloatIn01(float& f)	{
	f = (f<0 ? 0 : (f>1 ? 1 : f));
}
Material::Material()	{
	amb[0]=amb[1]=amb[2]=0.25f;amb[3]=1;
	dif[0]=dif[1]=dif[2]=0.85f;dif[3]=1;
	spe[0]=spe[1]=spe[2]=0.0f;spe[3]=1;
	shi = 0.f;//40.f/128.f;
	emi[0]=emi[1]=emi[2]=0.0f;emi[3]=1;
}
Material::Material(float r,float g,float b,bool useSpecularColor,float ambFraction,float speAddOn,float shininess)	{
	amb[0]=r*ambFraction;amb[1]=g*ambFraction;amb[2]=b*ambFraction;amb[3]=1;
	dif[0]=r;dif[1]=g;dif[2]=b;dif[3]=1;
	spe[3]=1;
	if (useSpecularColor)	{
		spe[0] = r + speAddOn;	clampFloatIn01(spe[0]);
		spe[1] = g + speAddOn;	clampFloatIn01(spe[1]);
		spe[2] = b + speAddOn;	clampFloatIn01(spe[2]);
		shi = shininess;		
	}
	else	{
		spe[0]=spe[1]=spe[2]=0;
		shi = 0.f;
	}
	emi[0]=emi[1]=emi[2]=0.0f;emi[3]=1;	
}	
void Material::reset() {*this = Material();}
void Material::bind() const	{
	#ifdef TAKE_IT_EASY
	glColor3fv(dif);
	#undef TAKE_IT_EASY
	#else //TAKE_IT_EASY
	#define GL_MY_MODE GL_FRONT
	//#define GL_MY_MODE GL_FRONT_AND_BACK
	
	glMaterialfv(GL_MY_MODE, GL_AMBIENT, amb);
	glMaterialfv(GL_MY_MODE, GL_DIFFUSE, dif);
	glMaterialfv(GL_MY_MODE, GL_SPECULAR, spe);
	glMaterialf(GL_MY_MODE, GL_SHININESS, (shi * 128.0f));
	glMaterialfv(GL_MY_MODE, GL_EMISSION, emi);
	
	#undef GL_MY_MODE
	#endif //TAKE_IT_EASY
}
void Material::displayDebugInfo() const	{
	printf("amb{%1.2f,%1.2f,%1.2f,%1.2f};\n",amb[0],amb[1],amb[2],amb[3]);
	printf("dif{%1.2f,%1.2f,%1.2f,%1.2f};\n",dif[0],dif[1],dif[2],dif[3]);
	printf("spe{%1.2f,%1.2f,%1.2f,%1.2f};\n",spe[0],spe[1],spe[2],spe[3]);
	printf("shi: %1.4f in[0,1];\n",shi);	
	printf("emi{%1.2f,%1.2f,%1.2f,%1.2f};\n",emi[0],emi[1],emi[2],emi[3]);
}
void Material::lighten(float q)	{
	for (int i=0;i<3;i++)	{
		amb[i]+=q;
		dif[i]+=q;
		spe[i]+=q;			
	}
	clamp();
}
void Material::darken(float q)	{
	for (int i=0;i<3;i++)	{
		amb[i]-=q;
		dif[i]-=q;
		spe[i]-=q;			
	}
	clamp();	
}
void Material::clamp()	{
	for (int i=0;i<4;i++)	{
		clampFloatIn01(amb[i]);
		clampFloatIn01(dif[i]);
		clampFloatIn01(spe[i]);
		clampFloatIn01(emi[i]);		
	}
	clampFloatIn01(shi);
}
//endregion



btTriangleMeshEx::btMeshSubPart::btMeshSubPart()	{reset();}
btTriangleMeshEx::btMeshSubPart::btMeshSubPart(int _useVertexArrayInSubpartIndex,int _useTexcoordArrayInSubpartIndex,bool _useIndsAsTexcoordInds)	{
	reset();
	useVertexArrayInSubpartIndex = _useVertexArrayInSubpartIndex;
	useTexcoordArrayInSubpartIndex = _useTexcoordArrayInSubpartIndex;
	useIndsAsTexcoordInds = _useIndsAsTexcoordInds;
}
void btTriangleMeshEx::btMeshSubPart::reset()	{
			verts.resize(0);
			inds.resize(0);
			useVertexArrayInSubpartIndex = -1;
			texcoords.resize(0);
			tinds.resize(0);
			useTexcoordArrayInSubpartIndex = -1;
			useIndsAsTexcoordInds = false;
			textureFilePath[0] = '\0';
			textureId = 0;
			material = Material();
}

btTriangleMeshEx::btTriangleMeshEx() : tiva(NULL),isTivaDirty(false)	{
	meshSubParts.reserve(MAX_NUMBER_SUBMESHES);	// MANDATORY: prevents pointers to subparts verts and inds to move in resizing operations (200 is probably too big anyway).
}
	
btTriangleMeshEx::~btTriangleMeshEx()	{
	if (tiva) {delete tiva;tiva=NULL;}	
}

btTriangleMeshEx::btTriangleMeshEx(const btTriangleMeshEx& c)	{
	*this = c;
}
const btTriangleMeshEx& btTriangleMeshEx::operator=(const btTriangleMeshEx& c)	{
	this->meshSubParts = *(const_cast < btAlignedObjectArray<btTriangleMeshEx::btMeshSubPart>* > (&c.meshSubParts));	//very big memory copy...
	if (tiva) {delete tiva;tiva = NULL;}
	return *this;
}

btTriangleMeshEx::operator btTriangleIndexVertexArrayPtr()	{
	if (tiva && !isTivaDirty) return tiva;
	isTivaDirty = false;
	if (tiva) {delete tiva;tiva=NULL;}
	tiva = new btTriangleIndexVertexArray();
	const unsigned char * lastValidVertexBase = NULL;
	int lastValidNumVertices = 0;
	bool reuseLastValidVertexBase;
	for (int subpart=0,subparts=meshSubParts.size();subpart<subparts;subpart++)	{
		const btMeshSubPart& subMesh = meshSubParts[subpart];
		
		btIndexedMesh mesh;
		mesh.m_numTriangles = subMesh.inds.size()/3;
   		mesh.m_triangleIndexBase = (const unsigned char *) &subMesh.inds[0];
   		mesh.m_triangleIndexStride = 3 * sizeof(int);
   		
   		if (!lastValidVertexBase) reuseLastValidVertexBase = false;
   		else reuseLastValidVertexBase = (subMesh.verts.size()==0 && (subMesh.useVertexArrayInSubpartIndex<0 || subMesh.useVertexArrayInSubpartIndex>=subparts || meshSubParts[subMesh.useVertexArrayInSubpartIndex].verts.size()==0));
   		if (!reuseLastValidVertexBase)	{
   			if (subMesh.verts.size()>0)	{
  				mesh.m_numVertices = lastValidNumVertices = subMesh.verts.size();
   				mesh.m_vertexBase = lastValidVertexBase = (const unsigned char *) &subMesh.verts[0].x(); 
   			}
   			else	{
   				mesh.m_numVertices = lastValidNumVertices = meshSubParts[subMesh.useVertexArrayInSubpartIndex].verts.size();
   				mesh.m_vertexBase = lastValidVertexBase = (const unsigned char *) &meshSubParts[subMesh.useVertexArrayInSubpartIndex].verts[0].x();  				
   			}	
   		}
   		else	{
   			mesh.m_numVertices = lastValidNumVertices;
   			mesh.m_vertexBase = lastValidVertexBase;   			
   		}
   			
   		mesh.m_vertexStride = sizeof(btVector3);

		tiva->addIndexedMesh(mesh,PHY_INTEGER);
	}
	return tiva;	
}

	
void btTriangleMeshEx::reset()	{
	meshSubParts.clear();
	isTivaDirty = true;
}

int btTriangleMeshEx::getNumSubParts(int optionalMeshIndex) const	{
	if (optionalMeshIndex<0) return meshSubParts.size();
	int meshCnt=-1;bool meshTracked = false;int baseSubPartIndex = -1;
	int numSubParts=0;
	for (int subpart = 0,subparts = meshSubParts.size();subpart<subparts;subpart++)	{
		const btMeshSubPart& m = meshSubParts[subpart];
		if (m.verts.size()>0) {
			++meshCnt;
			if (optionalMeshIndex == meshCnt) {baseSubPartIndex = subpart;meshTracked = true;++numSubParts;}
			else if (optionalMeshIndex > meshCnt) meshTracked = false;
			continue;
		}	
		if ((baseSubPartIndex>=0 && m.useVertexArrayInSubpartIndex==baseSubPartIndex) || (m.useVertexArrayInSubpartIndex==-1 && meshTracked)) ++numSubParts;	
	}
	return numSubParts;
}
int btTriangleMeshEx::getBaseSubPartIndex(int optionalMeshIndex) const	{
	if (optionalMeshIndex<0) return -1;//meshSubParts.size() > 0 ? 0 : -1;
	int meshCnt=-1;
	for (int subpart = 0,subparts = meshSubParts.size();subpart<subparts;subpart++)	{
		const btMeshSubPart& m = meshSubParts[subpart];
		if (m.verts.size()>0) {
			++meshCnt;
			if (optionalMeshIndex == meshCnt) return subpart;
		}	
	}
	return meshSubParts.size();	
}
btAlignedObjectArray< int > btTriangleMeshEx::getSubpartIndices(int optionalMeshIndex) const	{
	btAlignedObjectArray< int > rv;
	rv.reserve(meshSubParts.size());
	if (optionalMeshIndex<0) {
		for (int subpart = 0,subparts = meshSubParts.size();subpart<subparts;subpart++)	{
			rv.push_back(subpart);
		}
		return rv;
	}	
	int meshCnt=-1;bool meshTracked = false;int baseSubPartIndex = -1;
	for (int subpart = 0,subparts = meshSubParts.size();subpart<subparts;subpart++)	{
		const btMeshSubPart& m = meshSubParts[subpart];
		if (m.verts.size()>0) {
			++meshCnt;
			if (optionalMeshIndex == meshCnt) {rv.push_back(subpart);baseSubPartIndex = subpart;meshTracked = true;}
			else if (optionalMeshIndex > meshCnt) meshTracked = false;
			continue;
		}	
		if ((baseSubPartIndex>=0 && m.useVertexArrayInSubpartIndex==baseSubPartIndex) || (m.useVertexArrayInSubpartIndex==-1 && meshTracked))rv.push_back(subpart);	
	}
	return rv;
}


btVector3 btTriangleMeshEx::GetAabbHalfExtents(btVector3* pOptionalCenterPointOut,int optionalMeshIndex) const	{	
	btVector3 aabbHalfExtents(0,0,0);
	if (pOptionalCenterPointOut) *pOptionalCenterPointOut = btVector3(0,0,0);
	
	btVector3 cmax(0,0,0);
	btVector3 cmin(0,0,0);
	
	bool noStartingVertex = true;
	const int optionalSubpartIndex = getBaseSubPartIndex(optionalMeshIndex);
	for (int subpart=0,subparts=meshSubParts.size();subpart<subparts;subpart++)	{
		if (optionalSubpartIndex>=0 && optionalSubpartIndex!=subpart) continue;	 
		const btMeshSubPart& m = meshSubParts[subpart];
		for (int i=0,sz=m.verts.size();i<sz;i++)	{
			const btVector3& v = m.verts[i];
			if (noStartingVertex)	{
				noStartingVertex = false;
				cmin = cmax = v;
			}
			else	{
					if 		(cmax.x()<v.x()) 	cmax.setX(v.x());
					else if (cmin.x()>v.x())	cmin.setX(v.x());
					if 		(cmax.y()<v.y()) 	cmax.setY(v.y());
					else if (cmin.y()>v.y())	cmin.setY(v.y());
					if 		(cmax.z()<v.z()) 	cmax.setZ(v.z());
					else if (cmin.z()>v.z())	cmin.setZ(v.z());														
			}			
		}
	}	
	
	aabbHalfExtents = (cmax - cmin)*btScalar(0.5);
	if (pOptionalCenterPointOut) {
		*pOptionalCenterPointOut =  (cmax + cmin)*btScalar(0.5);
	}	
	return aabbHalfExtents;
}
const btVector3 btTriangleMeshEx::getAabbHalfExtents(int optionalMeshIndex) const	{
	return GetAabbHalfExtents(NULL,optionalMeshIndex);
}
const btVector3 btTriangleMeshEx::getAabbFullExtents(int optionalMeshIndex) const	{
	return GetAabbHalfExtents(NULL,optionalMeshIndex)*btScalar(2.);
}
const btVector3 btTriangleMeshEx::getCenterPoint(int optionalMeshIndex) const	{
	btVector3 cp;
	GetAabbHalfExtents(&cp,optionalMeshIndex);
	return cp;
}

void btTriangleMeshEx::shift(const btVector3& shiftFactor,int optionalMeshIndex)	{
	const int optionalSubpartIndex = getBaseSubPartIndex(optionalMeshIndex);
	for (int subpart=0,subparts=meshSubParts.size();subpart<subparts;subpart++)	{
		if (optionalSubpartIndex>=0 && optionalSubpartIndex!=subpart) continue;	 
		btMeshSubPart& m = meshSubParts[subpart];
		for (int i=0,sz=m.verts.size();i<sz;i++)	{
			btVector3& v = m.verts[i];
			v+=shiftFactor;
		}
	}		
}
void btTriangleMeshEx::scale(const btVector3& scaling,int optionalMeshIndex)	{
	const int optionalSubpartIndex = getBaseSubPartIndex(optionalMeshIndex);
	for (int subpart=0,subparts=meshSubParts.size();subpart<subparts;subpart++)	{
		if (optionalSubpartIndex>=0 && optionalSubpartIndex!=subpart) continue;	 
		btMeshSubPart& m = meshSubParts[subpart];
		for (int i=0,sz=m.verts.size();i<sz;i++)	{
			btVector3& v = m.verts[i];
			v*=scaling;
		}
	}
}
void btTriangleMeshEx::rotate(const btMatrix3x3& mt,int optionalMeshIndex)	{
	const int optionalSubpartIndex = getBaseSubPartIndex(optionalMeshIndex);
	for (int subpart=0,subparts=meshSubParts.size();subpart<subparts;subpart++)	{
		if (optionalSubpartIndex>=0 && optionalSubpartIndex!=subpart) continue;	 
		btMeshSubPart& m = meshSubParts[subpart];
		for (int i=0,sz=m.verts.size();i<sz;i++)	{
			btVector3& v = m.verts[i];
			v = mt*v;
		}
	}	
} 
void btTriangleMeshEx::rotate(const btQuaternion& q,int optionalMeshIndex)	{
	const btMatrix3x3 m(q);
	rotate(m,optionalMeshIndex);
}
void btTriangleMeshEx::transform(const btTransform& T,int optionalMeshIndex)	{
	const int optionalSubpartIndex = getBaseSubPartIndex(optionalMeshIndex);
	for (int subpart=0,subparts=meshSubParts.size();subpart<subparts;subpart++)	{
		if (optionalSubpartIndex>=0 && optionalSubpartIndex!=subpart) continue;	 
		btMeshSubPart& m = meshSubParts[subpart];
		for (int i=0,sz=m.verts.size();i<sz;i++)	{
			btVector3& v = m.verts[i];
			v = T.getOrigin() + T.getBasis()*v;
		}
	}	
} 
btVector3 btTriangleMeshEx::center(int  optionalMeshIndex)	{
	btVector3 center;
	GetAabbHalfExtents(&center,optionalMeshIndex);
	shift(-center,optionalMeshIndex);
	return center;	
}
btVector3 btTriangleMeshEx::setHeight(btScalar height,int  optionalMeshIndex)	{
	btVector3 ext = GetAabbHalfExtents(NULL,optionalMeshIndex) * btScalar(2.);
	const btScalar checkZero(SIMD_EPSILON);
	if (ext.y()<checkZero) ext.setY(checkZero);
	const btScalar uniformScalingFactor = height/ext.y();
	btVector3 scalingFactor(uniformScalingFactor,uniformScalingFactor,uniformScalingFactor);
	scale(scalingFactor,optionalMeshIndex);
	return scalingFactor;
}
btVector3 btTriangleMeshEx::setWidth(btScalar width,int  optionalMeshIndex)	{
	btVector3 ext = GetAabbHalfExtents(NULL,optionalMeshIndex) * btScalar(2.);
	const btScalar checkZero(SIMD_EPSILON);
	if (ext.x()<checkZero) ext.setX(checkZero);
	const btScalar uniformScalingFactor = width/ext.x();
	btVector3 scalingFactor(uniformScalingFactor,uniformScalingFactor,uniformScalingFactor);
	scale(scalingFactor,optionalMeshIndex);	
	return scalingFactor;
}
btVector3 btTriangleMeshEx::setLength(btScalar length,int  optionalMeshIndex)	{
	btVector3 ext = GetAabbHalfExtents(NULL,optionalMeshIndex) * btScalar(2.);
	const btScalar checkZero(SIMD_EPSILON);
	if (ext.z()<checkZero) ext.setZ(checkZero);
	const btScalar uniformScalingFactor = length/ext.z();
	btVector3 scalingFactor(uniformScalingFactor,uniformScalingFactor,uniformScalingFactor);
	scale(scalingFactor,optionalMeshIndex);
	return scalingFactor;
}
btVector3 btTriangleMeshEx::setFullExtents(const btVector3& extents,int  optionalMeshIndex)	{
	btVector3 ext = GetAabbHalfExtents(NULL,optionalMeshIndex) * btScalar(2.);
	const btScalar checkZero(SIMD_EPSILON);
	if (ext.x()<checkZero) ext.setX(checkZero);
	if (ext.y()<checkZero) ext.setY(checkZero);
	if (ext.z()<checkZero) ext.setZ(checkZero);
	btVector3 scalingFactor(extents.x()/ext.x(),extents.y()/ext.y(),extents.z()/ext.z());
	scale(scalingFactor,optionalMeshIndex);
	return scalingFactor;
}
btVector3 btTriangleMeshEx::setHalfExtents(const btVector3& extents,int  optionalMeshIndex)	{
	btVector3 ext = GetAabbHalfExtents(NULL,optionalMeshIndex);
	const btScalar checkZero(SIMD_EPSILON);
	if (ext.x()<checkZero) ext.setX(checkZero);
	if (ext.y()<checkZero) ext.setY(checkZero);
	if (ext.z()<checkZero) ext.setZ(checkZero);
	btVector3 scalingFactor(extents.x()/ext.x(),extents.y()/ext.y(),extents.z()/ext.z());
	scale(scalingFactor,optionalMeshIndex);
	return scalingFactor;
}

#include <stdio.h>

void btTriangleMeshEx::displayDebugInfo() const	{
	printf("Num meshSubParts: %1d\n",meshSubParts.size());
	for (int i=0;i<meshSubParts.size();i++)	{
		printf("submesh: %1d. ",i);
		const btMeshSubPart& m = meshSubParts[i];
		if (m.verts.size()>0) printf("numVerts: %1d ",m.verts.size());
		printf("numTri: %1d ",m.inds.size()/3);
		if (m.verts.size()==0 && m.useVertexArrayInSubpartIndex>=0) printf("referring to verts in submesh %1d ",m.useVertexArrayInSubpartIndex);
		
		if (m.texcoords.size()>0) printf("numTexcoords: %1d ",m.texcoords.size());
		if (m.tinds.size()>0) {
			printf("numTexTri: %1d ",m.tinds.size()/3);
			if (m.texcoords.size()==0 && m.useTexcoordArrayInSubpartIndex>=0) printf("referring to texcoords in submesh %1d",m.useTexcoordArrayInSubpartIndex);
		}
		if (m.texcoords.size()>0 || m.tinds.size()>0) {
			if (m.useIndsAsTexcoordInds) printf(" useIndsAsTexcoordInds = true");
			//else printf(" useIndsAsTexcoordInds = false");
		}	
		printf("\n");		
		const int len = strlen(m.textureFilePath);
		if (len>0 || m.textureId>0) printf("\ttexId=%d	path: \"%s\"\n",m.textureId,m.textureFilePath);
		//m.material.displayDebugInfo();
	}
}

bool btTriangleMeshEx::glDrawMeshSubPart(int subpartIndex,const btVector3& scaling,const bool useFlatNormals) const	{
	const int subparts = meshSubParts.size();
	if (subpartIndex<0 || subpartIndex>=subparts) return false;
	const btMeshSubPart& m = meshSubParts[subpartIndex];
	
	const btAlignedObjectArray < btVector3 >* pverts = &m.verts;
	const btAlignedObjectArray < btVector2 >* ptexcoords = &m.texcoords;
	const btAlignedObjectArray < int >* pinds = &m.inds;
	const btAlignedObjectArray < int >* ptinds = &m.tinds;
	
	int useAt = m.useVertexArrayInSubpartIndex;
	if (useAt!=subpartIndex)	{
		if (useAt<0 || useAt>=subparts)	{
			for (int j = subpartIndex-1;j>=0;j--)	{
				if (meshSubParts[j].verts.size()>0) {pverts = &meshSubParts[j].verts;break;}
			}
		}	
		else pverts = &meshSubParts[useAt].verts;	
	}
	useAt = m.useTexcoordArrayInSubpartIndex;
	if (useAt!=subpartIndex)	{
		if (useAt<0 || useAt>=subparts)	{
			for (int j = subpartIndex-1;j>=0;j--)	{
				if (meshSubParts[j].texcoords.size()>0) {ptexcoords = &meshSubParts[j].texcoords;break;}
			}
		}	
		else ptexcoords = &meshSubParts[useAt].texcoords;	
	}
	bool useTexcoords = true;
	if (m.useIndsAsTexcoordInds) {
		ptinds = &m.inds;
		if (ptexcoords->size()>0 && pverts->size()>ptexcoords->size()) {
			//printf("ptexcoords->size()>0 && pverts->size()>ptexcoords->size() ERROR in btTriangleMeshEx::glDrawMeshSubPart(%d)\n",subpartIndex);
			//return false;
			// Error, but we use fallback here (no textures)
			useTexcoords = false;
		}	
	}	
	else if (ptinds->size()>0 && pinds->size()>ptinds->size()) {
		//printf("ptinds->size()>0 && pinds->size()!=ptinds->size() ERROR in btTriangleMeshEx::glDrawMeshSubPart(%d)\n",subpartIndex);			
		//return false;
		// Error, but we use fallback here (no textures)
		useTexcoords = false;		
	}	
	
	const bool useVerts = pinds->size()>0 && pverts->size()>0;	
	if (!useVerts) {
		//printf("!(pinds->size()>0 && pverts->size()>0) ERROR in btTriangleMeshEx::glDrawMeshSubPart(%d)\n",subpartIndex);
		return false;
	}	
	useTexcoords = useTexcoords && ptinds->size()>0 && ptexcoords->size()>0;	
	
	const btAlignedObjectArray < btVector3 >& verts = *pverts;
	const btAlignedObjectArray < btVector2 >& texcoords = *ptexcoords;
	const btAlignedObjectArray < int >& inds = *pinds;
	const btAlignedObjectArray < int >& tinds = *ptinds;
	
	/*
	printf("bool btTriangleMeshEx::glDrawMeshSubPart(%d)\n",subpartIndex);
	printf("\tverts: %d  ",verts.size());
	printf("inds: %d\n",inds.size());
	if (useTexcoords)	{
		printf("\ttexcoords: %d  ",texcoords.size());
		printf("tinds: %d\n",tinds.size());
	}	
	*/
		
		
	btAlignedObjectArray < btVector3 > norms;
	glBegin(GL_TRIANGLES);
				if (useFlatNormals)	{
					for (int f = 0;f < inds.size();f=f+3)	{
						const int& id1 = inds[f];
						const int& id2 = inds[f+1];
						const int& id3 = inds[f+2];
					
						const btVector3& v0 = verts[id1];
						const btVector3& v1 = verts[id2];
						const btVector3& v2 = verts[id3];
					
						btVector3 v21(v2-v1);
						btVector3 v01(v0-v1);
						v21 = v21.cross(v01);
						v21.normalize();
						glNormal3fv(&v21.x());
						if (useTexcoords)	{
							const int& tid1 = tinds[f];
							const int& tid2 = tinds[f+1];
							const int& tid3 = tinds[f+2];
					
							const btVector2& t0 = texcoords[tid1];
							const btVector2& t1 = texcoords[tid2];
							const btVector2& t2 = texcoords[tid3];
							
							glTexCoord2fv(&t0.x());
							glVertex3f(v0.x()*scaling.x(),v0.y()*scaling.y(),v0.z()*scaling.z());
							glTexCoord2fv(&t1.x());
							glVertex3f(v1.x()*scaling.x(),v1.y()*scaling.y(),v1.z()*scaling.z());
							glTexCoord2fv(&t2.x());
							glVertex3f(v2.x()*scaling.x(),v2.y()*scaling.y(),v2.z()*scaling.z());
						}	
						else {
							glVertex3f(v0.x()*scaling.x(),v0.y()*scaling.y(),v0.z()*scaling.z());
							glVertex3f(v1.x()*scaling.x(),v1.y()*scaling.y(),v1.z()*scaling.z());
							glVertex3f(v2.x()*scaling.x(),v2.y()*scaling.y(),v2.z()*scaling.z());							
						}				
				}									
				}
				else	{
				//printf("Calculate norms (verts size:%d inds size:%d)\n",verts.size(),inds.size());
				norms.resize(verts.size());
				
				// Calculate vertex normals
				{
				for (int i=0;i<verts.size();i++)	norms[i] = btVector3(0,0,0);
				
				for (int f = 0;f < inds.size();f=f+3)	{
					const int& id1 = inds[f];
					const int& id2 = inds[f+1];
					const int& id3 = inds[f+2];
					
					const btVector3& v0 = verts[id1];
					const btVector3& v1 = verts[id2];
					const btVector3& v2 = verts[id3];
					
					btVector3 v21(v2-v1);
					btVector3 v01(v0-v1);
					v21 = v21.cross(v01);
					
					norms[id1]+=v21;
					norms[id2]+=v21;
					norms[id3]+=v21;					
				}
				for (int i=0;i<verts.size();i++)	{
					//if (norms[i].length2()>0.00001f) 
					norms[i].normalize();
					//else norms[i] = btVector3(0,1,0);	//Well, this normal is not used in this subpart, but I don't like leaving it at zero...
				}	
				}
				
				for (int i=0;i<inds.size();i++)	{
					glNormal3fv(&norms[inds[i]].x());
					if (useTexcoords)	{
						const btVector2& t = texcoords[tinds[i]];
						glTexCoord2fv(&t.x());
					}
					const btVector3& v = verts[inds[i]];
					glVertex3f(v.x()*scaling.x(),v.y()*scaling.y(),v.z()*scaling.z());
				}						
				}	
	glEnd();
	return true;
}

void btTriangleMeshEx::createMeshFootPrint(btTriangleMeshExFootPrint*& p,const btVector3& scaling, const bool useFlatNormals,const btAlignedObjectArray< const Material* >* pOptionalMaterialOverrideArray,const btAlignedObjectArray< GLuint >* pOptionalTextureOverrideArray,const btAlignedObjectArray< int >* pOptionalSubmeshIndexArray)	const{
	//if (p) DeleteMeshFootPrint(p);
	p = new btTriangleMeshExFootPrint();
	const int subparts = pOptionalSubmeshIndexArray ? pOptionalSubmeshIndexArray->size() : meshSubParts.size();
	p->reserve(subparts);
	for (int i =0;i < subparts;i++)	{
		int subpart = i;
		if (pOptionalSubmeshIndexArray) subpart = (*pOptionalSubmeshIndexArray)[i];
		if (subpart<0 || subpart>=meshSubParts.size()) continue;
		//printf("subpart: %d\n",subpart);		
		const btMeshSubPart& m = meshSubParts[subpart];
		btMeshSubPartFootPrint fp;
		
		fp.material = m.material;	
		if (pOptionalMaterialOverrideArray && pOptionalMaterialOverrideArray->size()>subpart &&  (*pOptionalMaterialOverrideArray)[subpart]!=NULL) fp.material = *((*pOptionalMaterialOverrideArray)[subpart]);
		//fp.material.displayDebugInfo();
		fp.textureId = m.textureId;
		if (pOptionalTextureOverrideArray && pOptionalTextureOverrideArray->size()>subpart &&  (*pOptionalTextureOverrideArray)[subpart]>0) fp.textureId = (*pOptionalTextureOverrideArray)[subpart];		
		
		fp.dl = glGenLists(1);
		if (fp.dl==0)  {
			printf("void btTriangleMeshEx::createMeshFootPrint(...) Error: dl == 0: ");			
			if (glGetError() == GL_INVALID_OPERATION) printf("GL_INVALID_OPERATION");
			printf("\n");
		}	
		glNewList(fp.dl,GL_COMPILE);
		const bool ok = glDrawMeshSubPart(subpart,scaling,useFlatNormals);
		glEndList();
		
		if (ok)	DisplayListHelper::Add(fp.dl);
		else	{
			glDeleteLists(fp.dl,1);
			fp.dl = 0;
		}
		//if (ok) printf("void btTriangleMeshEx::createMeshFootPrint(...) OK DL[%d].\n",subpart);
		//else printf("void btTriangleMeshEx::createMeshFootPrint(...) !OK DL[%d].\n",subpart);
		p->push_back(fp);
	}
	btTriangleMeshExFootPrint::Helper::Add(p);	
}
/*
void btTriangleMeshEx::DeleteMeshFootPrint(btTriangleMeshExFootPrint*& p)	{
	if (p)	{
		p->resize(0);
		delete p;
		p = NULL;
	}
}
*/

void btTriangleMeshExFootPrint::glDraw(const btTriangleMeshExFootPrint::Overrides* ovr,const btTriangleMeshExFootPrint::Overrides* ovrFallback,const bool useAmbientLightOnly) const	{
	bool useScaling(false);					// valid for all submeshes.
	bool useSpecularOnTexture(true);
        bool isGlNormalizeEnabled(false);

	#define GL_LIGHT_MODEL_COLOR_CONTROL_EXT 0x81F8
	#define GL_SINGLE_COLOR_EXT 0x81F9	
	#define GL_SEPARATE_SPECULAR_COLOR_EXT 0x81FA   		
	
	
	glPushMatrix();
	if (ovr) {
                const btVector3& offset = ovr->offset;
                //glTranslatef(offset.x(),offset.y(),offset.z());
                const btVector3& scaling = ovr->scaling;
		useScaling = (scaling.x()!=1 || scaling.y()!=1 || scaling.z()!=1);
		if (useScaling)	glScalef(scaling.x(),scaling.y(),scaling.z());
		useSpecularOnTexture = !ovr->disableSpecularOnTexture;
	}
	if (ovrFallback)	{
                const btVector3& offset = ovrFallback->offset;
		glTranslatef(offset.x(),offset.y(),offset.z());	
                const btVector3& scaling = ovrFallback->scaling;
		useScaling = useScaling || (scaling.x()!=1 || scaling.y()!=1 || scaling.z()!=1);
		if (useScaling)	glScalef(scaling.x(),scaling.y(),scaling.z());	
		if (useSpecularOnTexture) useSpecularOnTexture = !ovrFallback->disableSpecularOnTexture;	
	}	
        //if (useScaling) glEnable(GL_NORMALIZE);
        useSpecularOnTexture = (useSpecularOnTexture && !useAmbientLightOnly);
	if (useSpecularOnTexture) glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL_EXT, GL_SEPARATE_SPECULAR_COLOR_EXT); 

	const bool bindMaterial = true;
	const bool bindTexture = true;

	for (int subpart=0,subparts = this->size();subpart<subparts;subpart++) {
		const btTriangleMeshEx::btMeshSubPartFootPrint& m = (*this)[subpart];
		if (m.dl==0) continue;
		
		GLuint texId = 0;
		if (bindTexture)	 {
			texId = m.textureId;
			if (ovrFallback) {
				const btTriangleMeshExFootPrint::Overrides* OV = ovrFallback;
				const btAlignedObjectArray< GLuint >& ta = OV->textureArray;
				if (ta.size()>subpart &&  ta[subpart]!=0) texId = ta[subpart];
			}
			if (ovr) {
				const btTriangleMeshExFootPrint::Overrides* OV = ovr;
				const btAlignedObjectArray< GLuint >& ta = OV->textureArray;
				if (ta.size()>subpart &&  ta[subpart]!=0) texId = ta[subpart];
			}		
			glBindTexture(GL_TEXTURE_2D,texId);		
		}
		
		if (bindMaterial)	{
			const Material* mat = &m.material;
			if (ovrFallback) {
				const btTriangleMeshExFootPrint::Overrides* OV = ovrFallback;
				const btAlignedObjectArray< const Material*>& ma = OV->materialArray;
				if (ma.size()>subpart &&  ma[subpart]!=NULL) mat = ma[subpart];
			}
			if (ovr) {
				const btTriangleMeshExFootPrint::Overrides* OV = ovr;
				const btAlignedObjectArray< const Material*>& ma = OV->materialArray;
				if (ma.size()>subpart &&  ma[subpart]!=NULL) mat = ma[subpart];
			}			
			if (useAmbientLightOnly && (!bindTexture || texId==0)) {
				glColor4fv(mat->amb);									
				//glColor3f(mat->amb[0]+0.25f*mat->dif[0],mat->amb[1]+0.25f*mat->dif[1],mat->amb[2]+0.25f*mat->dif[2]);
			}	
			else mat->bind();
		}		

                if (useScaling) {
                    bool avoidGlNormalizeCallsWhenScaling(false);
                    if (ovrFallback) avoidGlNormalizeCallsWhenScaling = ovrFallback->avoidGlNormalizeCallsWhenScaling;
                    if (ovr) avoidGlNormalizeCallsWhenScaling = ovr->avoidGlNormalizeCallsWhenScaling;

                    if (isGlNormalizeEnabled)   {
                        if (avoidGlNormalizeCallsWhenScaling) {
                            glDisable(GL_NORMALIZE);
                            isGlNormalizeEnabled = false;
                        }
                    }
                    else if (!avoidGlNormalizeCallsWhenScaling) {
                        glEnable(GL_NORMALIZE);
                        isGlNormalizeEnabled = true;
                    }
                }

		glCallList(m.dl);			
	}
        if (useScaling && isGlNormalizeEnabled)	glDisable(GL_NORMALIZE);
	glPopMatrix();
	
	if (useSpecularOnTexture) glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL_EXT, GL_SINGLE_COLOR_EXT); 
		
}

void btTriangleMeshExFootPrint::Overrides::displayDebugInfo() const {
	printf("offset:		(%1.2f,%1.2f,%1.2f)\n",offset.x(),offset.y(),offset.z());
	printf("scaling:	(%1.2f,%1.2f,%1.2f)\n",scaling.x(),scaling.y(),scaling.z());
	if (disableSpecularOnTexture) printf("disableSpecularOnTexture: true\n");
	else printf("disableSpecularOnTexture: false\n");
        if (avoidGlNormalizeCallsWhenScaling) printf("avoidGlNormalizeCallsWhenScaling: true\n");
	for (int i=0;i<materialArray.size();i++)	{
		 const Material* mat = materialArray[i];
		 if (!mat) continue;
		 printf("------materialArray[%d]------\n",i);
		 mat->displayDebugInfo();
		 if (i == materialArray.size()-1) printf("-----------------------------\n");		 
	}
	for (int i=0;i<textureArray.size();i++)	{
		 const GLuint texId = textureArray[i];
		 if (texId>0) printf("textureArray[&d] = %d\n",i,texId);		 		 
	}	
}


btTriangleMeshExFootPrint::Overrides::Overrides(const btTriangleMeshExFootPrint::Overrides& ovr)	{*this = ovr;}
const btTriangleMeshExFootPrint::Overrides& btTriangleMeshExFootPrint::Overrides::operator=(const btTriangleMeshExFootPrint::Overrides& ovr)	{
	//*this=ovr;	// recursive in all paths
	this->offset = ovr.offset;
	this->scaling = ovr.scaling;
	this->disableSpecularOnTexture = ovr.disableSpecularOnTexture;
        this->avoidGlNormalizeCallsWhenScaling = ovr.avoidGlNormalizeCallsWhenScaling;
	this->materialArray = *(const_cast < btAlignedObjectArray < const Material* >* > (&ovr.materialArray));
	this->textureArray = *(const_cast < btAlignedObjectArray < GLuint >* > (&ovr.textureArray));
	
	return *this;
}
void btTriangleMeshExFootPrint::Overrides::reset() {*this=btTriangleMeshExFootPrint::Overrides();}

 //region Mesh Loading Methods
	
#include <stdlib.h>
#include <iostream>
#include <fstream>

#ifndef USE_STRING_CONVERSION_MODE_NUMBER
#define USE_STRING_CONVERSION_MODE_NUMBER 2	// in [1,3] (3 is the best, but needs <boost/lexical_cast.hpp>, 2 is the second best choice)
#endif //USE_STRING_CONVERSION_MODE_NUMBER

#if (USE_STRING_CONVERSION_MODE_NUMBER==2)
#include <sstream>
#elif  (USE_STRING_CONVERSION_MODE_NUMBER==3)
#include <boost/lexical_cast.hpp>	// Just 2 lines of code use it... TODO: find replecement
#endif //USE_STRING_CONVERSION_MODE_NUMBER

class String {
public:
inline static std::vector<std::string> Split(const std::string& text,const char c=' ')	{
std::string tex(text);
std::vector<std::string> ret;
std::size_t beg=std::string::npos;
while ( (beg=tex.find_first_of(c))!=std::string::npos)
	{
	ret.push_back(tex.substr(0,beg));
	tex=tex.substr(beg+1);
	}
ret.push_back(tex);	
return ret;	
}
inline static std::string TrimStart(const std::string& text,const char c=' ')	{
std::string tex(text);
while (tex.size()>0 && tex[0]==c) tex=tex.substr(1);
return tex;
}
inline static std::string TrimEnd(const std::string& text,const char c=' '){
std::string tex(text);
while (tex.size()>0 && tex[tex.size()-1]==c) tex=tex.substr(0,tex.size()-1);
return tex;
}
inline static std::string Trim(const std::string& text,const char c=' ')	{
	return TrimStart(TrimEnd(text),c);
}
inline static std::string TrimStart(const std::string& text,const char c1,const char c2)	{
	std::string tex(text);
	char c;
	while (tex.size()>0)	{	 	
	 	c = tex[0];
	 	if (c==c1 || c==c2) {tex=tex.substr(1);continue;}
	 	break;
	 }
	return tex;
}
inline static std::string TrimEnd(const std::string& text,const char c1,const char c2)	{
	std::string tex(text);
	char c;
	while (tex.size()>0)	{	 	
	 	c = tex[tex.size()-1];
	 	if (c==c1 || c==c2) {tex=tex.substr(0,tex.size()-1);continue;}
	 	break;
	 }	 
	return tex;
}
inline static std::string Trim(const std::string& text,const char c1,const char c2)	{
	return TrimStart(TrimEnd(text,c1,c2),c1,c2);
}

inline static bool StartsWith(const std::string& text,const std::string& start){
unsigned long textLength=text.size();
unsigned long searchLength=start.size();
if (searchLength>textLength) return false;
for (int t=0;t<searchLength;t++)
	{
	if (start[t]!=text[t]) return false;
	}
return true;	
}
inline static std::string Replace(const std::string&  baseText,const std::string& textToReplace,const std::string& replacement) {
std::string text(baseText);
if (textToReplace=="") return text;
std::size_t textToReplaceLength=textToReplace.size();
std::size_t i=std::string::npos;
while ( (i=text.find(textToReplace,0))!=std::string::npos )
	{
   	text=text.substr(0,i)+replacement+text.substr(i+textToReplaceLength);
   	}
return text;
}
inline static std::string ToLower(const std::string& text)	{
std::string ret(text);
for(int i=0;i<ret.length();++i) ret[i]=tolower(ret[i]);
return ret;	
}
inline static std::string GetExtension(const std::string& filePath){
std::size_t beg=filePath.find_last_of('.');
if (beg!=std::string::npos)  return ToLower(filePath.substr(beg));
return "";
}
inline static bool FileExists(const std::string& filePath) {
	FILE * fid = fopen(filePath.c_str(), "r");
	if (!fid) return false;
	fclose(fid);	
	return true;
}
inline static bool Break(const std::string& text,const char breaker,std::string& part0,std::string& part1,bool findFirst=true,int keepBreakerInPart=-1)	{
	part0=part1="";
	std::size_t beg=std::string::npos;
	if (findFirst) beg = text.find(breaker);
	else beg = text.find_last_of(breaker);
	if (beg==std::string::npos) return false;
	if (keepBreakerInPart == 0)	{
		part0 = text.substr(0,beg+1);
		part1 = text.substr(beg+1);	
	}
	else if (keepBreakerInPart == 1)	{
		part0 = text.substr(0,beg);
		part1 = text.substr(beg);	
	}
	else	{
		part0 = text.substr(0,beg);
		part1 = text.substr(beg+1);
	}
	return true;
}
inline static std::string Extract(const std::string& text,const char startChar,const char endChar,bool* optionalOperationSuccessfullFlag=NULL)	{
	std::string rv("");
	std::size_t beg=std::string::npos;
	beg = text.find(startChar);
	if (beg == std::string::npos) {
		if (optionalOperationSuccessfullFlag) *optionalOperationSuccessfullFlag = false;
		return rv;
	}	
	std::size_t beg2=std::string::npos;
	beg2 = text.find(endChar,beg+1);
	if (beg2 == std::string::npos) {
		if (optionalOperationSuccessfullFlag) *optionalOperationSuccessfullFlag = false;
		return rv;
	}
	rv = text.substr(beg+1,beg2-(beg+1));
	if (optionalOperationSuccessfullFlag) *optionalOperationSuccessfullFlag = true;
	//printf("Line: \"%s\"\nExtracted: \"%s\"\n",text.c_str(),rv.c_str());		
	return rv;
}

#if  (USE_STRING_CONVERSION_MODE_NUMBER==2 || USE_STRING_CONVERSION_MODE_NUMBER==3)
template < typename T > inline static T Convert(const std::string& text)	{
	#if  (USE_STRING_CONVERSION_MODE_NUMBER==2)
	static std::locale classicLocale(std::locale::classic());
	std::istringstream s(text);s.imbue(classicLocale);	// (some locales map the decimal separator with the comma, instead of with the dot).
    T t;
   	s >> t;
   	if (s.fail()) {
	   	#ifndef __GNUC__
    	std::runtime_error e("Invalid string conversion of: "+text);
        throw e;
        #endif //__GNUC__
    }
   	return t;
	#elif  (USE_STRING_CONVERSION_MODE_NUMBER==3)
	return boost::lexical_cast< T > (text);
	#endif //USE_STRING_CONVERSION_MODE_NUMBER
}
#endif //USE_STRING_CONVERSION_MODE_NUMBER

inline static int ConvertToInt(const std::string& text)	{
	#if (USE_STRING_CONVERSION_MODE_NUMBER==1)
	return atoi(text);
	#else //USE_STRING_CONVERSION_MODE_NUMBER
	return Convert< int >(text);
	#endif //USE_STRING_CONVERSION_MODE_NUMBER
}
inline static float ConvertToFloat(const std::string& text)	{
	#if (USE_STRING_CONVERSION_MODE_NUMBER==1)
	return atof(text);
	#else //USE_STRING_CONVERSION_MODE_NUMBER
	return Convert< float >(text);
	#endif //USE_STRING_CONVERSION_MODE_NUMBER
}	

protected:
String() {}
};	

class Path {
	public:
	static std::string GetDirectoryName(const std::string& filePath) {
		if (filePath[filePath.size()-1]==':') return filePath;
		std::size_t beg=filePath.find_last_of('\\');
		std::size_t beg2=filePath.find_last_of('/');
		beg= (beg==std::string::npos) ? beg2 : (beg2==std::string::npos) ? beg : (beg>beg2?beg:beg2);
		if (beg!=std::string::npos) return filePath.substr(0,beg);
		return "";	
	}
	static char GetSlash() {
		#if (defined(_WIN32) || defined(_WIN64))
		return '\\';
		#else
		return '/';
		#endif
	}
	protected:
	Path() {}
};


// I just want to export this to HACDDemo.cpp
bool FileExists(const char* filePath)	{
	FILE * fid = fopen(filePath, "r");
	if (!fid) return false;
	fclose(fid);	
	return true;
}
// This too:
bool FileHasExtension(const char* filePath,const char* extensionString)	{
	const std::string fp = filePath;
	const std::string ext = extensionString;
	return (String::GetExtension(filePath)==String::ToLower(ext));
}
// This too
size_t GetExeFileDirectoryPlusRelativeFilePath(char* pBuffer, const size_t bufferSize,const char* exeFileFullPath,const char* relativeFilePath)	{
	if (!exeFileFullPath || !relativeFilePath || !pBuffer) 	return 0;
	std::string path = exeFileFullPath;
	path = Path::GetDirectoryName(path);
        if (path.size()==0) path=".";
        path+=Path::GetSlash();
	path+=relativeFilePath;
	if (path.size()>=bufferSize-1) return path.size()+1;
	for (int i=0;i<path.size();i++) pBuffer[i] = path[i];
	pBuffer[path.size()] = '\0';
	return 0;
}

#undef USE_STRING_CONVERSION_MODE_NUMBER

bool btTriangleMeshEx::loadFromFile(const char* filePath,bool invertYZ)	{
	const std::string path = filePath;
	const std::string ext = String::GetExtension(path);
	if (ext==".obj") return loadFromOBJFile(filePath,invertYZ);
	else if (ext==".off") return loadFromOFFFile(filePath,invertYZ);
	#ifdef LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE
	else if (ext==".bcs" || ext==".bullet") return loadFromMeshInterfaceInsideBCSFile(filePath,invertYZ);	
	#endif//LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE
	else printf("Error while trying to load file: %s\n. Format %s is not supported.\n",filePath,ext.c_str());
	return false;
}	


static void AddMaterial(std::map < std::string,Material >& materialMap,std::map < std::string,std::string >& textureMap,const std::string& path,const std::string& curDir)	{
if (!FileExists(path.c_str())) return;

const float dif2ambFactor = 0.5f;	// MUST BE <=1. If no ambient color is provided, obtain one by: amb = dif * dif2ambFactor;


const char tabChar = 9;
std::string line="";
std::ifstream in(path.c_str(),std::ios::in);
if (!in) {
	printf("File path: \"%s\" doesn't exist. File not found.\n",path.c_str());
	return;
}	
try 
{
int numb=0;	
Material material;std::string matName="";bool ambMissing=true;;std::string matTexturePath="";float opacity = 1.0f;bool firstTime = true;
while (std::getline(in,line)) 
{
line=String::Trim(String::TrimEnd(String::TrimEnd(line,'\n'),'\r'),' ',tabChar);
if (String::StartsWith(line,"newmtl "))
	{
	if (!firstTime && matName.size()>0) {
		if (ambMissing) {material.amb[0] = dif2ambFactor * material.dif[0];material.amb[1] = dif2ambFactor * material.dif[1];material.amb[2] = dif2ambFactor * material.dif[2];}
		materialMap[matName]=material;
		if (matTexturePath.size()>0) textureMap[matName] = matTexturePath;
	}	
	if (firstTime) firstTime = false;
	line=String::Trim(line.substr(7));
	material.reset();opacity = 1.0f;material.shi = 115.0f/128.0f;
	matName=line;matTexturePath="";ambMissing=true;
	numb++;
	continue;
	}	
else if (String::StartsWith(line,"Ns "))  //Don't know what it is
	{
	line=String::Trim(line.substr(3));
	float temp=String::ConvertToFloat(line);
	material.shi = temp < 0 ? 0 : temp > 1000 ? 1 : temp/1000.0f;
	continue;
	}
else if (String::StartsWith(line,"Kd "))  //Diffuse Light
	{
	std::vector<std::string> split=String::Split(String::Trim(line.substr(3)));
	float temp[4];temp[3]=opacity;
	for (int t=0;t<split.size();t++) if (t<4) temp[t]=String::ConvertToFloat(split[t]);
	const int splitSize = split.size();
	if (splitSize == 2) temp[2] = temp[1];
	else if (splitSize == 1) temp[2] = temp[1] = temp[0];
	for (int t=0;t<4;t++) 	{
		temp[t]=(temp[t]>1?1:(temp[t]<0?0:temp[t]));
	}
	material.dif[0]=temp[0];
	material.dif[1]=temp[1];
	material.dif[2]=temp[2];
	material.dif[3]=temp[3];
	continue;
	}	
else if (String::StartsWith(line,"Ka "))  //Ambient Light
	{
	std::vector<std::string> split=String::Split(String::Trim(line.substr(3)));
	float temp[4];temp[3]=opacity;
	for (int t=0;t<split.size();t++) if (t<4) temp[t]=String::ConvertToFloat(split[t]);
	const int splitSize = split.size();
	if (splitSize == 2) temp[2] = temp[1];
	else if (splitSize == 1) temp[2] = temp[1] = temp[0];
	for (int t=0;t<4;t++) 	{
		temp[t]=(temp[t]>1?1:(temp[t]<0?0:temp[t]));
	}
	material.amb[0]=temp[0];
	material.amb[1]=temp[1];
	material.amb[2]=temp[2];
	material.amb[3]=temp[3];
	ambMissing=false;
	continue;
	}	
else if (String::StartsWith(line,"Ks "))  //Specular Light
	{//vertex
	std::vector<std::string> split=String::Split(String::Trim(line.substr(3)));
	float temp[4];temp[3]=opacity;
	for (int t=0;t<split.size();t++) if (t<4) temp[t]=String::ConvertToFloat(split[t]);
	const int splitSize = split.size();
	if (splitSize == 2) temp[2] = temp[1];
	else if (splitSize == 1) temp[2] = temp[1] = temp[0];
	for (int t=0;t<4;t++) 	{
		temp[t]=(temp[t]>1?1:(temp[t]<0?0:temp[t]));
	}
	material.spe[0]=temp[0];
	material.spe[1]=temp[1];
	material.spe[2]=temp[2];
	material.spe[3]=temp[3];
	continue;
	}		
else if (String::StartsWith(line,"d "))   //Opacity 
	{//vertex
	line=String::Trim(line.substr(2));
	float temp=String::ConvertToFloat(line);
	opacity=temp=(temp>1?1:(temp<0?0:temp));	
	//if (opacity<1.0) material.isTransparent = 1;
	//material.amb[3] = material.dif[3] = material.spe[3] = material.emi[3] = opacity; 
	//material.amb[3] = material.dif[3] = opacity;	
	material.dif[3] = opacity;	
	continue;
	}
else if (String::StartsWith(line,"illum "))  //Don't know what it is, but I've set it so that when < 0 (not used by the .obj format): material.isDoubleFace = material.useAlphaTest = true;
	{//vertex
	/*
	line=String::Trim(line.substr(6));
	int temp=StringConvertToInt(line);
	if (temp<0) {
		material.isDoubleFace = true;	//This is just a hack...
		material.useAlphaTest = true;	//This is just another hack...
	}	
	*/
	continue;
	}	
else if (String::StartsWith(line,"map_Kd ")) 
	{//vertex
	line=String::Trim(String::Trim(String::Trim(line.substr(7),'\n'),'\r'),' ');
	std::string relPath=line;
	std::string texturePath= curDir + line;
	const std::string originalTexturePath = texturePath;
	if (!FileExists(texturePath.c_str())) {
		if (!FileExists(relPath.c_str()))	{
			printf("File: \"%s\" is not present.\n",relPath.c_str());
			continue;
		}
		else matTexturePath = relPath;
	}
	else matTexturePath = originalTexturePath;
	continue;
	}
}
if (numb>0 && matName.size()>0) {
		if (ambMissing) {material.amb[0] = dif2ambFactor * material.dif[0];material.amb[1] = dif2ambFactor * material.dif[1];material.amb[2] = dif2ambFactor * material.dif[2];}
		materialMap[matName]=material;
		if (matTexturePath.size()>0) textureMap[matName] = matTexturePath;
}	
}
catch (...) 
{
}	
in.close();

}	
bool btTriangleMeshEx::loadFromOBJFile(const char* filePath,bool invertYZ)	{
	const char tabChar = 9;
	std::ifstream in(filePath,std::ios::in);
	if (!in) return false;
	size_t numVerts=0,numTexCoords=0;std::string lastObjectName("");
	bool isFirstMesh = true;
	const int startSubpartIndex = meshSubParts.size();
	int currentSubpartIndex = startSubpartIndex;
	meshSubParts.push_back(btMeshSubPart());
	btAlignedObjectArray < btVector3 >& verts =  meshSubParts[startSubpartIndex].verts;
	btAlignedObjectArray < btVector2 >& texCoords =  meshSubParts[startSubpartIndex].texcoords;
	unsigned long numPolygonLines=0,numTriFaces=0,numQuadFaces=0,numPolyFaces=0;
	std::map < std::string, Material > materialMap;
	std::map < std::string, std::string > textureMap;
	try {
		std::string line="";bool firstObject = true;unsigned long numLines=0,lastObjectLine=0;
 		while (std::getline(in,line))	{
 			++numLines;
 			//printf("%s\n",line.c_str());
 			 //region Parser
			line=String::Trim(String::TrimEnd(line,'\n','\r'),' ',tabChar);			
			if (String::StartsWith(line,"mtllib "))	{//mtllib file	
				std::string mtllibPath=Path::GetDirectoryName(filePath);
				mtllibPath+=Path::GetSlash();
				const std::string dir = mtllibPath;
				mtllibPath+=String::Trim(line.substr(7));
				if (!FileExists(mtllibPath.c_str())) {
					printf("File path: \"%s\" doesn't exist. File not found.\n",mtllibPath.c_str());
				}		
				else AddMaterial(materialMap,textureMap,mtllibPath,dir);
				continue;
			}
			else if (String::StartsWith(line,"o "))	{//object
				lastObjectName=String::Trim(line.substr(2));	// new part nme (unused)
				lastObjectLine = numLines;
				continue;
			}	
			else if (String::StartsWith(line,"v "))	{//vertex
				std::vector<std::string> split=String::Split( String::Trim(line.substr(2)) );
				btVector3 v;
				v.setX(String::ConvertToFloat(String::Trim(split[0])));
				if (!invertYZ)	{
					v.setY(String::ConvertToFloat(String::Trim(split[1])));
					v.setZ(String::ConvertToFloat(String::Trim(split[2])));
				}
				else	{
					v.setZ(-String::ConvertToFloat(String::Trim(split[1])));
					v.setY(String::ConvertToFloat(String::Trim(split[2])));					
				}	
				verts.push_back(v);
				numVerts++;
				continue;
			}
			else if (String::StartsWith(line,"vt "))	{//texture
				std::vector<std::string> split=String::Split( String::Trim(line.substr(2)) );
				btVector2 t;
				t.setX(String::ConvertToFloat(String::Trim(split[0])));
				t.setY(String::ConvertToFloat(String::Trim(split[1])));
				texCoords.push_back(t);
				numTexCoords++;
				continue;
			}	
			else if (String::StartsWith(line,"vn "))	{//normal	
				continue;
			}		
			else if (String::StartsWith(line,"usemtl "))	{
				if (isFirstMesh) isFirstMesh = false;
				else	{
					currentSubpartIndex = meshSubParts.size();
					meshSubParts.push_back(btMeshSubPart(startSubpartIndex));
				}
				const std::string matName=String::Trim(line.substr(7));
				{
				std::map< std::string,Material >::const_iterator it = materialMap.find(matName);
				if (it!=materialMap.end()) meshSubParts[currentSubpartIndex].material = it->second;
                                else printf("Material \"%s\" not found.\n",matName.c_str());
				}
				{
				std::map< std::string,std::string >::const_iterator it = textureMap.find(matName);
				if (it!=textureMap.end()) {
					const std::string& name = it->second;
					if (name.size()<=1024)	{
						char* c = &meshSubParts[currentSubpartIndex].textureFilePath[0];
						for (int i=0;i<name.size();i++) c[i] = name[i];
						c[name.size()]='\0';
						meshSubParts[currentSubpartIndex].textureId = TextureHelper::Get(c);	
					}	
				}	
				}
				
				continue;
			}		
			else if (String::StartsWith(line,"f "))	{
				btMeshSubPart& mesh = meshSubParts[currentSubpartIndex];
				std::vector<std::string> face=String::Split( String::Trim(line.substr(2)),' ' );
				const size_t faceSize = face.size();
				#pragma region Face Processing (To rewrite parsing /)
				const int tcPlace = 1;	// V,T,N
				if (faceSize==3)	{	
					for (size_t t=0;t<face.size();t++)	{
						int ind=0,Tind=0;//,Nind=0;			
						bool indOK = numVerts>0, TindOK = numTexCoords>0;//, NindOK = numNorms>0;			
						std::vector<std::string> part=String::Split(String::Trim(face[t]),'/');
						const size_t partsSize = part.size();
						if (partsSize==0) {
							printf(".obj parsing error. %s has a face with 0 indices:\n\n%s\n\n",filePath,line.c_str());
							meshSubParts.resize(startSubpartIndex);
							in.close();
							return false;
						}
						//inds
						try 	{
							if (indOK && partsSize>0 && part[0].size()>0) {
								ind=String::ConvertToInt(String::Trim(part[0],' ',tabChar))-1;
								if (ind<0) ind+=numVerts+1;
								if (ind<0) {
									printf(".obj parsing error. %s has a face with a negative index: %d\n\n%s\n\n",filePath,ind,line.c_str());
									meshSubParts.resize(startSubpartIndex);
									return false;
								}
							}
							else indOK = false;	
						}
						catch (...){
							printf(".obj parsing error. %s exception thrown while parsing line:\n\n%s\n\n",filePath,line.c_str());
							meshSubParts.resize(startSubpartIndex);
							in.close();
							return false;
						}
						//mesh.inds.push_back(ind);
						meshSubParts[currentSubpartIndex].inds.push_back(ind);						
						//tinds:
						try 	{
							if (TindOK && partsSize>tcPlace && part[tcPlace].size()>0) {									
								Tind=String::ConvertToInt(String::Trim(part[tcPlace],' ',tabChar))-1;
								if (Tind<0) Tind+=numTexCoords+1;
								if (Tind<0) {
									printf(".obj parsing error. %s has a face with a negative texture index: %d\n\n%s\n\n",filePath,Tind,line.c_str());
									meshSubParts.resize(startSubpartIndex);
									in.close();
									return false;
								}
								meshSubParts[currentSubpartIndex].tinds.push_back(Tind);
							}
							else TindOK = false;	
						}
						catch (...){
							printf(".obj parsing error. %s exception thrown while parsing line:\n\n%s\n\n",filePath,line.c_str());
							meshSubParts.resize(startSubpartIndex);
							in.close();
							return false;
						}
						
					}
					//printf("f3	%1d		%1d		%1d\n",mesh.inds[mesh.inds.size()-3],mesh.inds[mesh.inds.size()-2],mesh.inds[mesh.inds.size()-1]);
					++numTriFaces;
				}
				else if (faceSize>=4) {
					int inds[6],Tinds[6];//,Ninds[6];				
					bool indOK = numVerts>0, TindOK = numTexCoords>0;//, NindOK = numNorms>0;	
					for (int t=0;t<4;t++) {
						std::vector<std::string> part=String::Split(String::Trim(face[t]),'/');
						const size_t partsSize = part.size();
						if (partsSize==0) {
							printf(".obj parsing error. %s has a face with 0 indices:\n\n%s\n\n",filePath,line.c_str());
							meshSubParts.resize(startSubpartIndex);
							return false;
						}
						try {
							int& ind = inds[t];//int& Nind = Ninds[t];int& Tind = Tinds[t];
							if (indOK && partsSize>0 && part[0].size()>0) {
								ind=String::ConvertToInt(String::Trim(part[0],' ',tabChar))-1;
								if (ind<0) ind+=numVerts+1;
								if (ind<0) {
									printf(".obj parsing error. %s has a face with a negative index: %d\n\n%s\n\n",filePath,ind,line.c_str());
									meshSubParts.resize(startSubpartIndex);
									return false;
								}
							}
							else indOK = false;	
							}
						catch (...){
							printf(".obj parsing error. %s exception thrown while parsing line:\n\n%s\n\n",filePath,line.c_str());
							meshSubParts.resize(startSubpartIndex);
							in.close();
							return false;
						}
						try {
							//int& ind = inds[t];//int& Nind = Ninds[t];
							int& Tind = Tinds[t];
							if (TindOK && partsSize>tcPlace && part[tcPlace].size()>0) {
								Tind=String::ConvertToInt(String::Trim(part[tcPlace],' ',tabChar))-1;
								if (Tind<0) Tind+=numVerts+1;
								if (Tind<0) {
									printf(".obj parsing error. %s has a face with a negative texture index: %d\n\n%s\n\n",filePath,Tind,line.c_str());
									meshSubParts.resize(startSubpartIndex);
									return false;
								}
							}
							else TindOK = false;	
							}
						catch (...){
							printf(".obj parsing error. %s exception thrown while parsing line:\n\n%s\n\n",filePath,line.c_str());
							meshSubParts.resize(startSubpartIndex);
							in.close();
							return false;
						}						
					}
					inds[4]=inds[0];
					inds[5]=inds[2];
					Tinds[4]=Tinds[0];
					Tinds[5]=Tinds[2];					
					for (int i=0;i<6;i++)	{
						//mesh.inds.push_back(inds[i]);
						meshSubParts[currentSubpartIndex].inds.push_back(inds[i]);
						meshSubParts[currentSubpartIndex].tinds.push_back(Tinds[i]);
					}	
					if (faceSize>4) {
						++numPolyFaces;
						printf("Line %1d: \"%s\" :only first 4 indices added\n",(int)numLines-1,line.c_str());	
					}	
					else ++numQuadFaces;
					//printf("f4	%1d		%1d		%1d\n",mesh.inds[mesh.inds.size()-6],mesh.inds[mesh.inds.size()-4],mesh.inds[mesh.inds.size()-4]);					
					//printf("f4	%1d		%1d		%1d\n",mesh.inds[mesh.inds.size()-3],mesh.inds[mesh.inds.size()-2],mesh.inds[mesh.inds.size()-1]);
				}	
				else if (faceSize==2) ++numPolygonLines;
				else printf("Line %1d: \"%s\" :skipped\n",(int)numLines-1,line.c_str());			
				#pragma endregion
				continue;
			}	
	 			
 			//endregion 
 		}	
	}
	catch(...)	{
		in.close();
		printf(".obj parsing error: can't load file %s Exception thrown\n\n",filePath);							
		meshSubParts.resize(startSubpartIndex);
		return false;
	}
	in.close();
	isTivaDirty = true;
	if (numPolygonLines>0) printf("Skipped %1d faces with just two indices.\n",(int)numPolygonLines);
	//printf("Obj file data: numTriFaces: %d, numQuadFaces: %d, numPolyFaces: %d\n",numTriFaces,numQuadFaces,numPolyFaces);
	return true;		
}

bool btTriangleMeshEx::loadFromOFFFile(const char* filePath,bool invertYZ)	{
	const char tabChar = 9;
	std::ifstream in(filePath,std::ios::in);
	if (!in) return false;
	const int startSubpartIndex = meshSubParts.size();
	int currentSubpartIndex = startSubpartIndex;
	meshSubParts.push_back(btMeshSubPart());
	btAlignedObjectArray < btVector3 >& verts =  meshSubParts[startSubpartIndex].verts;
	int numVertices=0;
	int numTriangles = 0;
	std::vector < std::string > parts;
	size_t partsSize;size_t numPolygonLines = 0;
	try {
		std::string line="";unsigned long numLines=0;
		if (!std::getline(in,line)) {
			printf(".off parsing error. \"%s\" is empty.\n",filePath,line.c_str());
			meshSubParts.resize(startSubpartIndex);
			return false;
		}
		++numLines;
		line=String::ToLower(String::Trim(String::TrimEnd(line,'\n','\r'),' ',tabChar));			
		if (!String::StartsWith(line,"off"))	{
			printf(".off parsing error. \"%s\": Format tag not present.\n",filePath);
			meshSubParts.resize(startSubpartIndex);
			return false;			
		}	
		if (!std::getline(in,line)) {
			printf(".off parsing error. \"%s\" is empty.\n",filePath,line.c_str());
			meshSubParts.resize(startSubpartIndex);
			return false;
		}
		++numLines;
		line=String::ToLower(String::Trim(String::TrimEnd(line,'\n','\r'),' ',tabChar));			
		parts = String::Split(line,' ');
		partsSize = parts.size();
		bool ok = false;
		if (partsSize>=2)	{
			ok = true;
			try	{
				numVertices = String::ConvertToInt(String::Trim(parts[0],' ',tabChar));
				numTriangles = String::ConvertToInt(String::Trim(parts[1],' ',tabChar));
			}
			catch(...)	{
				ok = false;
			}
		}
		if (!ok)	{
			printf(".off parsing error. \"%s\": I Can't read the number of vertices/triangles.\n",filePath);
			meshSubParts.resize(startSubpartIndex);
			return false;			
		}		
		meshSubParts[startSubpartIndex].verts.reserve(numVertices);
		meshSubParts[startSubpartIndex].inds.reserve(numTriangles*3);
		size_t cntLine = 0;		
 		while (std::getline(in,line))	{
 			++numLines;
 			++cntLine;
 			//printf("%s\n",line.c_str());
 			 //region Parser
			line=String::Trim(String::TrimEnd(line,'\n','\r'),' ',tabChar);			
			if (cntLine<=numVertices)	{//vertex
				std::vector<std::string> split=String::Split( line );
				btVector3 v;
				v.setX(String::ConvertToFloat(String::Trim(split[0])));
				if (!invertYZ)	{
					v.setY(String::ConvertToFloat(String::Trim(split[1])));
					v.setZ(String::ConvertToFloat(String::Trim(split[2])));
				}
				else	{
					v.setZ(-String::ConvertToFloat(String::Trim(split[1])));
					v.setY(String::ConvertToFloat(String::Trim(split[2])));					
				}	
				verts.push_back(v);
				continue;
			}
			else //if (cntLine<=numVertices+numIndices)	
			{
				btMeshSubPart& mesh = meshSubParts[currentSubpartIndex];								
                                std::vector<std::string> face=String::Split( String::Trim(line/*.substr(2)*/),' ' );
				const size_t faceSize = face.size();
				int numIndices(0);
				if (faceSize>1)	{
					try	{
						numIndices=String::ConvertToInt(String::Trim(face[0],' ',tabChar));	
					}
					catch(...){
							printf(".off parsing error. %s exception thrown while parsing line:\n\n%s\n\n",filePath,line.c_str());
							meshSubParts.resize(startSubpartIndex);
							return false;					
					}
				}
				#pragma region Face Processing
                                if ((numIndices==3 && faceSize>=4) || (numIndices==4 && faceSize==4))	{
					for (size_t t=1;t<4;t++)	{
						int ind=0;							
						try 	{
							ind=String::ConvertToInt(String::Trim(face[t],' ',tabChar));
							if (ind<0) {
								printf(".off parsing error. %s has a face with a negative index:\n\n%s\n\n",filePath,line.c_str());
								meshSubParts.resize(startSubpartIndex);
								return false;

							}
						}
						catch (...){
							printf(".off parsing error. %s exception thrown while parsing line:\n\n%s\n\n",filePath,line.c_str());
							meshSubParts.resize(startSubpartIndex);
							return false;
						}
						meshSubParts[currentSubpartIndex].inds.push_back(ind);						
					}
					//printf("f3	%1d		%1d		%1d\n",mesh.inds[mesh.inds.size()-3],mesh.inds[mesh.inds.size()-2],mesh.inds[mesh.inds.size()-1]);
				}
                                else if (numIndices>=4 && faceSize>=numIndices+1) {
                                        int inds[3];
                                        // First triangle:
                                        for (int t=1;t<4;t++) {
                                            int& ind = inds[t-1];
                                            try {
                                                                ind=String::ConvertToInt(String::Trim(face[t],' ',tabChar));
                                                                if (ind<0) {
                                                                        printf(".off parsing error. %s has a face with a negative index:\n\n%s\n\n",filePath,line.c_str());
                                                                        meshSubParts.resize(startSubpartIndex);
                                                                        return false;
                                                                }
                                                        }
                                            catch (...){
                                                        printf(".off parsing error. %s exception thrown while parsing line:\n\n%s\n\n",filePath,line.c_str());
                                                        meshSubParts.resize(startSubpartIndex);
                                                        return false;
                                            }
                                            meshSubParts[currentSubpartIndex].inds.push_back(ind);
                                        }
                                        // Successive points:
                                        for (int t=4;t<numIndices+1;t++)  {
                                            int& ind = inds[1];
                                            try {
                                                            ind=String::ConvertToInt(String::Trim(face[t],' ',tabChar));
                                                            if (ind<0) {
                                                                    printf(".off parsing error. %s has a face with a negative index:\n\n%s\n\n",filePath,line.c_str());
                                                                    meshSubParts.resize(startSubpartIndex);
                                                                    return false;
                                                            }
                                                    }
                                            catch (...){
                                                    printf(".off parsing error. %s exception thrown while parsing line:\n\n%s\n\n",filePath,line.c_str());
                                                    meshSubParts.resize(startSubpartIndex);
                                                    return false;
                                            }
                                            meshSubParts[currentSubpartIndex].inds.push_back(inds[2]);
                                            meshSubParts[currentSubpartIndex].inds.push_back(inds[1]);
                                            meshSubParts[currentSubpartIndex].inds.push_back(inds[0]);
                                            inds[2]=inds[1];   //inds[1] is lost here: next triangle won't use it
                                        }
                                }
				else if (numIndices==2) ++numPolygonLines;
				else printf("Line %1d: \"%s\" :skipped\n",(int)numLines-1,line.c_str());			
				#pragma endregion
				continue;
			}	
	 			
 			//endregion 
 		}	
	}
	catch(...)	{
		in.close();
		printf(".off parsing error: can't load file %s Exception thrown\n\n",filePath);							
		meshSubParts.resize(startSubpartIndex);
		return false;
	}
	in.close();
	isTivaDirty = true;
	if (numPolygonLines>0) printf("Skipped %1d faces with just two indices.\n",(int)numPolygonLines);
	return true;		
}

bool btTriangleMeshEx::saveToOFFFile(const char* filePath)	{
	std::ofstream out(filePath,std::ios::out);
	if (!out) return false;
	
	out<<"OFF"<<std::endl;
	const int subparts=meshSubParts.size();
	int numVerts=0,numInds=0;
	btAlignedObjectArray < int > indsBases;
	indsBases.resize(subparts);
	for (int subpart=0;subpart<subparts;subpart++)	{
		const btMeshSubPart& subMesh = meshSubParts[subpart];
		const btAlignedObjectArray < btVector3 >& verts = subMesh.verts;
		const btAlignedObjectArray < int >& inds = subMesh.inds;
		indsBases[subpart]=numVerts;
		numVerts+=verts.size();
		numInds+=inds.size();
	}	
	out<<numVerts<<" "<<numInds/3<<" 0"<<std::endl;
	for (int subpart=0;subpart<subparts;subpart++)	{
		const btMeshSubPart& subMesh = meshSubParts[subpart];		
		const btAlignedObjectArray < btVector3 >& verts = subMesh.verts;
		for (int i=0,sz=verts.size();i<sz;i++)	{
			const btVector3& v = verts[i];
			out<<v.x()<<" "<<v.y()<<" "<<v.z()<< std::endl;
		}			
	}		
	for (int subpart=0;subpart<subparts;subpart++)	{
		const btMeshSubPart& subMesh = meshSubParts[subpart];
		const btAlignedObjectArray < int >& inds = subMesh.inds;
		int indsBase = -1;
		if (subMesh.useVertexArrayInSubpartIndex>=0 && subMesh.useVertexArrayInSubpartIndex<subparts && meshSubParts[subMesh.useVertexArrayInSubpartIndex].verts.size()>0) {
			indsBase = indsBases[subMesh.useVertexArrayInSubpartIndex];
		}
		if (indsBase==-1)	{
			if (subMesh.verts.size()>0) indsBase = indsBases[subpart];
			else	{
				for (int p=subpart-1;p>=0;p--)	{
					if (meshSubParts[p].verts.size()>0) {
						indsBase = indsBases[p];
						break;
					}
				}				
			}
		}	
		//printf("INDSBASE[%d]=%d",subpart,indsBase);
		if (indsBase<0) indsBase=0;	
		for (int i=0,sz=inds.size();i<sz;i=i+3)	{
			out<<"3 "<<inds[i]+indsBase<<" "<<inds[i+1]+indsBase<<" "<<inds[i+2]+indsBase<< std::endl;
		}
	}		
	out.close();
	return true;
}


#ifdef INCLUDE_MESHES_FROM_APPGIMPACTTESTDEMO
 //region Default Meshes
bool btTriangleMeshEx::loadDefaultBunnyMesh()	{
	#include "BunnyMesh.inl"
	meshSubParts.push_back(btMeshSubPart());
	btMeshSubPart& mesh = meshSubParts[meshSubParts.size()-1];
		
	const int numVerts = BUNNY_NUM_VERTICES;//(sizeof(gVerticesBunny)/sizeof(gVerticesBunny[0])/3;
	mesh.verts.reserve(numVerts);
	int i3;	
	for (int i =0;i<numVerts;i++)	{
		i3 = 3*i;
		mesh.verts.push_back(btVector3(gVerticesBunny[i3],gVerticesBunny[i3+1],gVerticesBunny[i3+2]));
	}
	
	const int numTriangles = BUNNY_NUM_TRIANGLES;// (sizeof(gIndicesBunny)/sizeof(gIndicesBunny[0]);
	mesh.inds.reserve(numTriangles*3);
	for (int i =0;i<numTriangles;i++)	{
		mesh.inds.push_back(gIndicesBunny[i][0]);
		mesh.inds.push_back(gIndicesBunny[i][1]);
		mesh.inds.push_back(gIndicesBunny[i][2]);
	}
	
	return true;
}	
bool btTriangleMeshEx::loadDefaultTorusMesh()	{
	#include "TorusMesh.inl"
	meshSubParts.push_back(btMeshSubPart());
	btMeshSubPart& mesh = meshSubParts[meshSubParts.size()-1];
		
	const int numVerts = NUM_VERTICES;//(sizeof(gVertices)/sizeof(gVertices[0])/3;
	mesh.verts.reserve(numVerts);
	int i3;	
	for (int i =0;i<numVerts;i++)	{
		i3 = 3*i;
		mesh.verts.push_back(btVector3(gVertices[i3],gVertices[i3+1],gVertices[i3+2]));
	}
	
	const int numTriangles = NUM_TRIANGLES;//(sizeof(gIndices)/sizeof(gIndices[0]);
	mesh.inds.reserve(numTriangles*3);
	for (int i =0;i<numTriangles;i++)	{
		mesh.inds.push_back(gIndices[i][0]);
		mesh.inds.push_back(gIndices[i][1]);
		mesh.inds.push_back(gIndices[i][2]);
	}
	
	return true;
}
//endregion 
#endif //INCLUDE_MESHES_FROM_APPGIMPACTTESTDEMO

#ifdef LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE
#include <BulletWorldImporter/btBulletWorldImporter.h>		// In "Bullet/Extra/Serialize". Needs link to: BulletWorldImporter.lib
#include <BulletCollision/CollisionShapes/btCompoundShape.h>
#include <BulletCollision/CollisionShapes/btStridingMeshInterface.h>
#include <BulletCollision/CollisionShapes/btBvhTriangleMeshShape.h>
#include <BulletCollision/CollisionShapes/btScaledBvhTriangleMeshShape.h>
#include <BulletCollision/Gimpact/btGImpactShape.h>
#include <BulletCollision/CollisionShapes/btPolyhedralConvexShape.h>
#include <BulletCollision/CollisionShapes/btConvexPolyhedron.h>
#include <BulletCollision/CollisionShapes/btConvexHullShape.h>
#include <BulletCollision/CollisionShapes/btShapeHull.h>
#include <LinearMath/btConvexHullComputer.h>

static btCollisionShape* LoadBCS(const char* filename,bool verbose)	{	
	btBulletWorldImporter loader(0);//don't store info into the world
	loader.setVerboseMode(verbose);
	if (!loader.loadFile(filename)) return NULL;
	
	//int	loader.getNumCollisionShapes() const;
	//btCollisionShape* loader.getCollisionShapeByIndex(int index);
	//btCollisionShape* loader.getCollisionShapeByName(const char* name);
	
	btCollisionShape* shape = NULL;
	if (loader.getNumCollisionShapes()>0) shape = loader.getCollisionShapeByIndex(0);
	
	//btCollisionShape* shape = loader.getCollisionShapeByName("BCS");
	
	//TODO: Cleaner way:
	// 1) Deep clone Collision Shape	
	// 2) loader.deleteAllData(); // deletes all (collision shapes included)
	// 3) return Deep cloned Collision Shape
	
	// Here we don't delete all data from the loader. (leaks?)
	return shape;
}
static const btStridingMeshInterface* GetStridingMeshInterface(const btCollisionShape* shape,btVector3& scaling,btTransform& T)	{
	// scaling and T are OUT arguments that must be reset by the caller
	const btStridingMeshInterface* smi(NULL);
			if (shape->getShapeType()==GIMPACT_SHAPE_PROXYTYPE)	{
				const btGImpactShapeInterface* gimi = static_cast < const btGImpactShapeInterface* > (shape);
				if (gimi && gimi->getGImpactShapeType()==CONST_GIMPACT_TRIMESH_SHAPE)	{
					const btGImpactMeshShape* gishape = static_cast < const btGImpactMeshShape* > (gimi);
					smi = gishape->getMeshInterface();
					scaling*=gishape->getLocalScaling();
				}
			}
			else if (shape->getShapeType()==TRIANGLE_MESH_SHAPE_PROXYTYPE)	{
				const btBvhTriangleMeshShape* bvhShape = static_cast < const btBvhTriangleMeshShape* > (shape);
					smi = bvhShape->getMeshInterface();
					scaling*=bvhShape->getLocalScaling();
			}
			else if (shape->getShapeType()==SCALED_TRIANGLE_MESH_SHAPE_PROXYTYPE)	{
				const btScaledBvhTriangleMeshShape* sbvhShape = static_cast < const btScaledBvhTriangleMeshShape* > (shape);
				if (sbvhShape)	{
					const btBvhTriangleMeshShape* bvhShape = static_cast < const btBvhTriangleMeshShape* > (sbvhShape->getChildShape());
					if (bvhShape)	{
						smi = bvhShape->getMeshInterface();
						scaling*=sbvhShape->getLocalScaling();		//This must be left outside 'considerShapeLocalScaling'
						scaling*=bvhShape->getLocalScaling();
					}	
				}	
			}
			if (shape->getShapeType()==COMPOUND_SHAPE_PROXYTYPE)	{
				const btCompoundShape* cs = static_cast < const btCompoundShape* > (shape);
				if (cs->getNumChildShapes()>0)	{
					scaling*=cs->getLocalScaling();
					const btCollisionShape* childShape(cs->getChildShape(0));
					const btTransform& CT(cs->getChildTransform(0));
					T = btTransform(T.getBasis() * CT.getBasis(),T.getOrigin()+T.getBasis()*(CT.getOrigin()*cs->getLocalScaling()));
					return GetStridingMeshInterface(childShape,scaling,T);
				}
			}
	return smi;
}
bool btTriangleMeshEx::loadFromMeshInterfaceInsideBulletCollisionShape(const btCollisionShape* shape,bool invertYZ)	{
	if (!shape) return false;
	btVector3 scaling(1,1,1);
	btTransform T(btTransform::getIdentity());			
	const btStridingMeshInterface* smi = GetStridingMeshInterface(shape,scaling,T);
	if (smi) return loadFromMeshInterface(smi,invertYZ,scaling!=btVector3(1,1,1)?&scaling:NULL,&T);	
	return false;
}
bool btTriangleMeshEx::loadFromMeshInterfaceInsideBCSFile(const char* bcsFilePath,bool invertYZ)	{
	const btCollisionShape* shape = LoadBCS(bcsFilePath,false);
	if (!shape) return false;
	return loadFromMeshInterfaceInsideBulletCollisionShape(shape,invertYZ);
}
bool btTriangleMeshEx::loadFromMeshInterface(const btStridingMeshInterface* sti,bool invertYZ,const btVector3* pOptionalScaling,const btTransform* pOptionalTransform)	{
	if (!sti) return false; 
	const int subparts = sti->getNumSubParts();
	if (sti->getNumSubParts()<=0) return false;
	
	const int startSubMeshIndex = meshSubParts.size();	
	
	{
	typedef btScalar vertexType;
	typedef long indexType;
								
	const unsigned char *pverts;int numVerts;	PHY_ScalarType type;int stride;
	const unsigned char *pinds;int indsStride;int numTriangles;PHY_ScalarType indsType;
	
	btAlignedObjectArray < const unsigned char * > vertexBaseOfProcessedSubMeshes;
	btAlignedObjectArray < int > startVertexIndexOfVertexBaseOfProcessedSubMeshes;
	//#define DONT_REUSE_SHARED_VERTEX_ARRAY	// problably safer, but much slower
	
	unsigned int vCnt=0;
	unsigned int tCnt=0;
		
									
	for (int subpart=0;subpart<subparts;subpart++)	{
		sti->getLockedReadOnlyVertexIndexBase(&pverts,numVerts,type,stride,&pinds,indsStride,numTriangles,indsType,subpart);
		
		int startVertexIndex = 0;
		bool vertexArrayIsShared = false;	

		meshSubParts.push_back(btMeshSubPart());
		const int curSubPartIndex = meshSubParts.size()-1;
		btMeshSubPart& mesh = meshSubParts[curSubPartIndex];
		mesh.useVertexArrayInSubpartIndex = mesh.useTexcoordArrayInSubpartIndex = curSubPartIndex;
		
		
		#ifndef DONT_REUSE_SHARED_VERTEX_ARRAY	
		{
			for (int i = 0,sz = vertexBaseOfProcessedSubMeshes.size();i<sz;i++)	{
				if (pverts == vertexBaseOfProcessedSubMeshes[i])	{
					startVertexIndex = startVertexIndexOfVertexBaseOfProcessedSubMeshes[i];
					vertexArrayIsShared = true;
					mesh.useVertexArrayInSubpartIndex = mesh.useTexcoordArrayInSubpartIndex = startSubMeshIndex+startVertexIndex;
					break;
				}
			} 
		
			vertexBaseOfProcessedSubMeshes.push_back(pverts);
			startVertexIndexOfVertexBaseOfProcessedSubMeshes.push_back(startVertexIndex);
		}
		#else //DONT_REUSE_SHARED_VERTEX_ARRAY
			#undef DONT_REUSE_SHARED_VERTEX_ARRAY
		#endif //DONT_REUSE_SHARED_VERTEX_ARRAY
		
		const indexType indsAdder = (indexType) startVertexIndex; 
				
		
		try	{
			
			if (!vertexArrayIsShared)				
			switch (type)	{
				case PHY_FLOAT:	{
					typedef float glVertex;
					const glVertex* verts = (const glVertex*) pverts; 
					const int vertexDeltaStrideInGLVertexUnits = stride <= sizeof(glVertex)*3 ? 0 : (stride-sizeof(glVertex)*3)/sizeof(glVertex); 			
					btVector3 tempVert;
					
					mesh.verts.resize(numVerts);
					for (size_t i = 0; i < numVerts;++i)	{
						btVector3& v(mesh.verts[i]);
						
						tempVert.setX((vertexType) *verts++);
						tempVert.setY((vertexType) *verts++);
						tempVert.setZ((vertexType) *verts++);
						
						if (pOptionalScaling) tempVert*=(*pOptionalScaling);
						if (pOptionalTransform) tempVert= pOptionalTransform->getOrigin()+pOptionalTransform->getBasis()*tempVert;
						
						if (!invertYZ)	v = tempVert;
						else	{
							v.setX(tempVert.x());
							v.setZ(-tempVert.y());
							v.setY(tempVert.z());						
						}	
						
						verts+=vertexDeltaStrideInGLVertexUnits;
					}					
			 	break;
		 		}
				case PHY_DOUBLE:{
					typedef double glVertex;
					const glVertex* verts = (const glVertex*) pverts; 
					const int vertexDeltaStrideInGLVertexUnits = stride <= sizeof(glVertex)*3 ? 0 : (stride-sizeof(glVertex)*3)/sizeof(glVertex); 			
					btVector3 tempVert;
					
					mesh.verts.resize(numVerts);
					for (size_t i = 0; i < numVerts;++i)	{
						btVector3& v(mesh.verts[i]);
						
						tempVert.setX((vertexType) *verts++);
						tempVert.setY((vertexType) *verts++);
						tempVert.setZ((vertexType) *verts++);
						
						if (pOptionalScaling) tempVert*=(*pOptionalScaling);
						if (pOptionalTransform) tempVert= pOptionalTransform->getOrigin()+pOptionalTransform->getBasis()*tempVert;
						
						if (!invertYZ)	v = tempVert;
						else	{
							v.setX(tempVert.x());
							v.setZ(-tempVert.y());
							v.setY(tempVert.z());						
						}	
						
						verts+=vertexDeltaStrideInGLVertexUnits;
					}
			 	break;
		 		}
				default:
				btAssert((type == PHY_FLOAT) || (type == PHY_DOUBLE));
			}
		

			switch (indsType)	{				
				case PHY_INTEGER:	{
			 		typedef int glIndex;
			 		const glIndex* inds = (const glIndex*) pinds; 
			 		const int indexDeltaStrideInGLIndexUnits = indsStride <= sizeof(indsStride)*3 ? 0 : (stride-sizeof(indsStride)*3)/sizeof(indsStride); 
			 		
			 		mesh.inds.resize(numTriangles*3);int i3;
					for (size_t i = 0; i < numTriangles;++i)	{
						i3 = 3*i;
						mesh.inds[i3] = (indexType) *inds++;
						mesh.inds[i3+1] = (indexType) *inds++;
						mesh.inds[i3+2] = (indexType) *inds++;
						
						inds+=indexDeltaStrideInGLIndexUnits;
					}
				 	break;
				}
				case PHY_SHORT:	{
					typedef unsigned short glIndex;
			 		const glIndex* inds = (const glIndex*) pinds;
			 		const int indexDeltaStrideInGLIndexUnits = indsStride <= sizeof(indsStride)*3 ? 0 : (stride-sizeof(indsStride)*3)/sizeof(indsStride); 			 		
			 		
			 		mesh.inds.resize(numTriangles*3);int i3;
					for (size_t i = 0; i < numTriangles;++i)	{
						i3 = 3*i;
						mesh.inds[i3] = (indexType) *inds++;
						mesh.inds[i3+1] = (indexType) *inds++;
						mesh.inds[i3+2] = (indexType) *inds++;
						
						inds+=indexDeltaStrideInGLIndexUnits;
					}					 		 
				 	break;
				}
				default:
			 	btAssert((indsType == PHY_INTEGER) || (indsType == PHY_SHORT));
			}
		}
		catch (...) {
			meshSubParts.resize(startSubMeshIndex);
		}
		
		sti->unLockReadOnlyVertexBase(subpart);
		}	
	
	}
	
	const bool ok = (startSubMeshIndex < meshSubParts.size());
	isTivaDirty = ok;
	return ok;
}
#endif //LOAD_MESHES_FROM_STRIDING_MESH_INTERFACE

bool btTriangleMeshEx::loadDefaultPlaneMesh(int axis,btScalar halfLength,btScalar halfHeight,btScalar texRepX,btScalar texRepY,btScalar texOffX,btScalar texOffY,bool twoSided)	{
	meshSubParts.push_back(btMeshSubPart());
	int curSubPartIndex = meshSubParts.size()-1;
	btMeshSubPart& mesh = meshSubParts[curSubPartIndex];
	mesh.useVertexArrayInSubpartIndex = mesh.useTexcoordArrayInSubpartIndex = curSubPartIndex;
	mesh.verts.resize(4);
	switch (axis)	{
		case 1:
			mesh.verts[0] = btVector3(0	,-halfHeight	,-halfLength);
			mesh.verts[1] = btVector3(0	,-halfHeight	,halfLength );
			mesh.verts[2] = btVector3(0	,halfHeight		,halfLength );
			mesh.verts[3] = btVector3(0	,halfHeight		,-halfLength);					
		break;
		case 3:
			mesh.verts[0] = btVector3(-halfLength	,-halfHeight	,0);
			mesh.verts[1] = btVector3(halfLength		,-halfHeight	,0);
			mesh.verts[2] = btVector3(halfLength		,halfHeight		,0);
			mesh.verts[3] = btVector3(-halfLength	,halfHeight		,0);				
		break;
		case -1:
			mesh.verts[0] = btVector3(0	,-halfHeight	,halfLength );
			mesh.verts[1] = btVector3(0	,-halfHeight	,-halfLength);
			mesh.verts[2] = btVector3(0	,halfHeight		,-halfLength);
			mesh.verts[3] = btVector3(0	,halfHeight		,halfLength );						
		break;
		case -2:
			mesh.verts[0] = btVector3(halfLength		,0	,halfHeight);
			mesh.verts[1] = btVector3(-halfLength	,0	,halfHeight);
			mesh.verts[2] = btVector3(-halfLength	,0	,+halfHeight	);
			mesh.verts[3] = btVector3(halfLength		,0	,+halfHeight	);					
		break;
		case -3:
			mesh.verts[0] = btVector3(halfLength	,-halfHeight	,0);
			mesh.verts[1] = btVector3(-halfLength,-halfHeight	,0);
			mesh.verts[2] = btVector3(-halfLength,halfHeight		,0);
			mesh.verts[3] = btVector3(halfLength	,halfHeight		,0);					
		break;
		default:
			mesh.verts[0] = btVector3(-halfLength	,0	,halfHeight);
			mesh.verts[1] = btVector3(halfLength		,0	,halfHeight);
			mesh.verts[2] = btVector3(halfLength		,0	,-halfHeight	);
			mesh.verts[3] = btVector3(-halfLength	,0	,-halfHeight	);					
	}
	mesh.texcoords.resize(4);
	const btScalar htrX(texRepX*btScalar(0.5));
	const btScalar htrY(texRepY*btScalar(0.5));
	texOffX+=btScalar(0.5);
	texOffY+=btScalar(0.5);	
	mesh.texcoords[0] = btVector2(texOffX-htrX	,texOffY-htrY	);
	mesh.texcoords[1] = btVector2(texOffX+htrX	,texOffY-htrY	);
	mesh.texcoords[2] = btVector2(texOffX+htrX	,texOffY+htrY	);
	mesh.texcoords[3] = btVector2(texOffX-htrX	,texOffY+htrY	);
	mesh.useIndsAsTexcoordInds = true;	// No need for mesh.tinds...
	mesh.inds.resize(6);
	mesh.inds[0] = 0;
	mesh.inds[1] = 1;
	mesh.inds[2] = 2;
	mesh.inds[3] = 2;
	mesh.inds[4] = 3;
	mesh.inds[5] = 0;
	/*
	mesh.tinds.resize(6);
	mesh.tinds[0] = 0;
	mesh.tinds[1] = 1;
	mesh.tinds[2] = 2;
	mesh.tinds[3] = 2;
	mesh.tinds[4] = 3;
	mesh.tinds[5] = 0;	
	*/
	if (twoSided)	{
		const btMeshSubPart& prevMesh = mesh;
		meshSubParts.push_back(btMeshSubPart());
		curSubPartIndex = meshSubParts.size()-1;
		mesh = meshSubParts[curSubPartIndex];
		mesh.useVertexArrayInSubpartIndex = curSubPartIndex-1;
		mesh.useTexcoordArrayInSubpartIndex = curSubPartIndex;
		
		mesh.useIndsAsTexcoordInds = true;	// No need for mesh.tinds...
		mesh.texcoords.resize(4);
		mesh.texcoords[0] = prevMesh.texcoords[1];
		mesh.texcoords[1] = prevMesh.texcoords[0];
		mesh.texcoords[2] = prevMesh.texcoords[3];
		mesh.texcoords[3] = prevMesh.texcoords[2];

		mesh.inds.resize(6);
		mesh.inds[0] = 1;
		mesh.inds[1] = 0;
		mesh.inds[2] = 3;
		mesh.inds[3] = 3;
		mesh.inds[4] = 2;
		mesh.inds[5] = 1;		
	}
	isTivaDirty = true;
	return true;
}
bool btTriangleMeshEx::loadDefaultBoxMesh(const btVector3& halfExtents,bool useTextures,bool useOneMaterialForEachFace,btScalar texRepX,btScalar texRepY,btScalar texOffX,btScalar texOffY,unsigned short numOfFacesToCreate)	{
	if (numOfFacesToCreate==0) return false;
	if (numOfFacesToCreate>6) numOfFacesToCreate = 6;
	
	meshSubParts.push_back(btMeshSubPart());
	int curSubPartIndex = meshSubParts.size()-1;
	const int startSubPartIndex = curSubPartIndex;
	btMeshSubPart& mesh = meshSubParts[curSubPartIndex];
	mesh.useVertexArrayInSubpartIndex = startSubPartIndex;
	if (useTextures) mesh.useTexcoordArrayInSubpartIndex = startSubPartIndex;
	
	
	mesh.verts.resize(8);
	const btScalar x(halfExtents.x());
	const btScalar y(halfExtents.y());
	const btScalar z(halfExtents.z());
	mesh.verts[0].setValue(-x, y, z);
	mesh.verts[1].setValue( x, y, z);
	mesh.verts[2].setValue( x, y,-z);
	mesh.verts[3].setValue(-x, y,-z);
	mesh.verts[4].setValue(-x,-y, z);
	mesh.verts[5].setValue( x,-y, z);
	mesh.verts[6].setValue( x,-y,-z);
	mesh.verts[7].setValue(-x,-y,-z);
	
	
	if (useTextures)	{
		mesh.texcoords.resize(4);	// 6x4
		const btScalar htrX(texRepX*btScalar(0.5));
		const btScalar htrY(texRepY*btScalar(0.5));
		texOffX+=btScalar(0.5);
		texOffY+=btScalar(0.5);	
		const btScalar minX(texOffX-htrX);
		const btScalar minY(texOffY-htrY);
		const btScalar maxX(texOffX+htrX);
		const btScalar maxY(texOffX+htrX);
		// Face								Vertex index:
		mesh.texcoords[0].setValue(minX,minY);	// 0
		mesh.texcoords[1].setValue(maxX,minY);	// 1
		mesh.texcoords[2].setValue(maxX,maxY);	// 2
		mesh.texcoords[3].setValue(minX,maxY);	// 3		
	}
	
	int* pInds(NULL);
	int* ptInds(NULL);
	if (!useOneMaterialForEachFace)	{
		mesh.inds.resize(3*2*numOfFacesToCreate);
		if (useTextures) mesh.tinds.resize(3*2*numOfFacesToCreate);
	}
	
	for (int i=0;i<numOfFacesToCreate;i++)	{
		if (useOneMaterialForEachFace)	{
			if (i!=0)	{
				// Create new submesh:
				meshSubParts.push_back(btMeshSubPart());
				curSubPartIndex = meshSubParts.size()-1;
				mesh = meshSubParts[curSubPartIndex];
				mesh.useVertexArrayInSubpartIndex = startSubPartIndex;
				if (useTextures) mesh.useTexcoordArrayInSubpartIndex = startSubPartIndex;				
				
				mesh.inds.resize(3*2);
				if (useTextures) mesh.tinds.resize(3*2);
				
				pInds = &(mesh.inds[0]);
				if (useTextures) ptInds = &(mesh.tinds[0]);
			}
			else	{
				pInds = &(mesh.inds[0]);
				if (useTextures) ptInds = &(mesh.tinds[0]);
			}		
		}
		else	{
			pInds = &(mesh.inds[3*2*i]);
			if (useTextures) ptInds = &(mesh.tinds[3*2*i]);				
		}
		
		switch (i)	{
			case 0:	
				// Top face
				pInds[0] = 0;
				pInds[1] = 1;
				pInds[2] = 2;
				pInds[3] = 2;
				pInds[4] = 3;
				pInds[5] = 0;	
			break;
			case 1:	
				// Left face
				pInds[0] = 7;
				pInds[1] = 4;
				pInds[2] = 0;
				pInds[3] = 0;
				pInds[4] = 3;
				pInds[5] = 7;	
			break;
			case 2:	
				// Right face
				pInds[0] = 5;
				pInds[1] = 6;
				pInds[2] = 2;
				pInds[3] = 2;
				pInds[4] = 1;
				pInds[5] = 5;	
			break;
			case 3:	
				// Bottom face
				pInds[0] = 7;
				pInds[1] = 6;
				pInds[2] = 5;
				pInds[3] = 5;
				pInds[4] = 4;
				pInds[5] = 7;	
			break;				
			case 4:	
				// Front face
				pInds[0] = 4;
				pInds[1] = 5;
				pInds[2] = 1;
				pInds[3] = 1;
				pInds[4] = 0;
				pInds[5] = 4;	
			break;
			case 5:	
				// Back face
				pInds[0] = 6;
				pInds[1] = 7;
				pInds[2] = 3;
				pInds[3] = 3;
				pInds[4] = 2;
				pInds[5] = 6;	
			break;					
			default:
			continue;
		}

		
		if (useTextures)	{		
			ptInds[0] = 0;
			ptInds[1] = 1;
			ptInds[2] = 2;
			ptInds[3] = 2;
			ptInds[4] = 3;
			ptInds[5] = 0;	
		}	
	}
	isTivaDirty = true;
	return true;
}




//endregion


 //region SallyIDE Text Editor Settings
//Caret=(0,105)
//Bookmarks=(129,708,959,1470,1491,1528,1548,2056,2189)
//endregion
